/*
**
**                              D E T A I L . C
**
**
**      Module:     routines for detail panel
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      09-OCT-1992 AV  Version X1.0, first definition
**      16-OCT-1992 AV  added support for tcp statistics
**      17-OCT-1992 AV  added help page
**      20-OCT-1992 AV  added generic asy support
**      21-OCT-1992 AV  partial redesign
**      21-OCT-1992 AV  added generic SCSI support
**      23-OCT-1992 AV  Version 1.0
**      25-OCT-1992 AV  allow long ps display whenever LINES > 26
**
*/

#include "config.h"

#include <curses.h>
#include <panel.h>

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"
#include "uidutil.h"

#if defined(MODULE_FAS)
#   include "fas.h"
#elif defined(MODULE_SAS)
#   include "sas.h"
#elif defined(MODULE_ASY)
#   include "asy.h"
#endif

#define DPT_NONE            0
#define DPT_PROC            1
#define DPT_PROC_LONG       2
#define DPT_EXTRA           3
#define DPT_HACK            4
#define DPT_STREAM          5
#define DPT_SERIAL          6
#define DPT_HARD            7
#define DPT_TCP             8
#define DPT_IP              9
#define DPT_HELP           10
#define DPT_SCSI           11
#define DPT_SLIP           12

extern PANEL    *pscr;
extern WINDOW   *wscr;

PANEL       *pdet;
WINDOW      *wdet = (WINDOW *) 0;

static u_char   detailPanelType = DPT_NONE;

/*
**
**  PROC support
**
*/
#define DETAIL_PROC_TLY \
    (LINES > 38 ? (full ? PER_SEC_TLY : PER_SEC_TLY + 15) : PER_SEC_TLY)

#define DETAIL_PROC_LENGTH  (LINES - DETAIL_PROC_TLY)

static void DetailProcInit (int full)
{
    pdet = MakePanel (DETAIL_PROC_LENGTH, COLS, DETAIL_PROC_TLY, 0, "ps");
    if (pdet == (PANEL *) 0)
	LeaveText ("cannot make detail panel", 1);

    show_panel (pdet);
    top_panel (pdet);
    wdet = panel_window (pdet);
    DspProc (wdet, 1);
}

static void DetailProcUpdate ()
{
    DspProc (wdet, 0);
}

/*
**
**  EXTRA support
**
*/
#define DETAIL_EXTRA_TLY        ((LINES > 38) ? (PER_SEC_TLY + 15) : PER_SEC_TLY)
#define DETAIL_EXTRA_LENGTH     (LINES - DETAIL_EXTRA_TLY)

static void DetailExtraInit ()
{
    pdet = MakePanel (DETAIL_EXTRA_LENGTH, COLS, DETAIL_EXTRA_TLY, 0, "ex");
    if (pdet == (PANEL *) 0)
	LeaveText ("cannot make detail panel", 1);

    show_panel (pdet);
    top_panel (pdet);
    wdet = panel_window (pdet);

    DspVar (wdet, 0, EXTRA1_TLX);
    DspBoot (wdet, 0, EXTRA2_TLX);
    DspTune (wdet, 0, EXTRA3_TLX);
    DspProcSums (wdet, 0, EXTRA4_TLX);
}

static void DetailExtraUpdate ()
{
    DspProcSums (wdet, 0, EXTRA4_TLX);
}

/*
**
**  HACK support
**
*/
#define DETAIL_HACK_TLY         ((LINES > 38) ? (PER_SEC_TLY + 15) : PER_SEC_TLY)
#define DETAIL_HACK_LENGTH      (LINES - DETAIL_HACK_TLY)

static void DetailHackInit ()
{
    pdet = MakePanel (DETAIL_HACK_LENGTH, COLS, DETAIL_HACK_TLY, 0, "hack");
    if (pdet == (PANEL *) 0)
	LeaveText ("cannot make detail panel", 1);

    show_panel (pdet);
    top_panel (pdet);
    wdet = panel_window (pdet);

    DspHackLiterals (wdet, 0, 0);
    DspHack (wdet, 0, 0);
}

static void DetailHackUpdate ()
{
    DspHack (wdet, 0, 0);
}

/*
**
**  SCSI support
**
*/
#define DETAIL_SCSI_TLY         ((LINES > 38) ? (PER_SEC_TLY + 15) : PER_SEC_TLY)
#define DETAIL_SCSI_LENGTH      (LINES - DETAIL_SCSI_TLY)

static void DetailScsiInit ()
{
    pdet = MakePanel (DETAIL_SCSI_LENGTH, COLS, DETAIL_SCSI_TLY, 0, "scsi");
    if (pdet == (PANEL *) 0)
	LeaveText ("cannot make detail panel", 1);

    show_panel (pdet);
    top_panel (pdet);
    wdet = panel_window (pdet);

    DspScsiLiterals (wdet, 0, 0);
    DspScsi (wdet, 0, 0);
}

static void DetailScsiUpdate ()
{
    DspScsi (wdet, 0, 0);
}

/*
**
**  HELP support
**
*/
#define DETAIL_HELP_TLY         ((LINES > 38) ? (PER_SEC_TLY + 15) : PER_SEC_TLY)
#define DETAIL_HELP_LENGTH      (LINES - DETAIL_HELP_TLY)

static void DetailHelpInit ()
{
    pdet = MakePanel (DETAIL_HELP_LENGTH, COLS, DETAIL_HELP_TLY, 0, "help");
    if (pdet == (PANEL *) 0)
	LeaveText ("cannot make detail panel", 1);

    show_panel (pdet);
    top_panel (pdet);
    wdet = panel_window (pdet);

    DspHelpLiterals (wdet, 0, 0);
    DspHelp (wdet, 0, 0);
}

static void DetailHelpUpdate ()
{
    DspHelp (wdet, 0, 0);
}

/*
**
**  SLIP support
**
*/
#define DETAIL_SLIP_TLY         ((LINES > 38) ? (PER_SEC_TLY + 15) : PER_SEC_TLY)
#define DETAIL_SLIP_LENGTH      (LINES - DETAIL_SLIP_TLY)

static void DetailSlipInit ()
{
    pdet = MakePanel (DETAIL_SLIP_LENGTH, COLS, DETAIL_SLIP_TLY, 0, "slip");
    if (pdet == (PANEL *) 0)
	LeaveText ("cannot make detail panel", 1);

    show_panel (pdet);
    top_panel (pdet);
    wdet = panel_window (pdet);

    DspSlipLiterals (wdet, 0, 0);
    DspSlip (wdet, 0, 0);
}

static void DetailSlipUpdate ()
{
    DspSlip (wdet, 0, 0);
}

/*
**
**  IP support
**
*/
#define DETAIL_IP_TLY       ((LINES > 38) ? (PER_SEC_TLY + 15) : PER_SEC_TLY)
#define DETAIL_IP_LENGTH    (LINES - DETAIL_IP_TLY)

static void DetailIpInit ()
{
    pdet = MakePanel (DETAIL_IP_LENGTH, COLS, DETAIL_IP_TLY, 0, "ip");
    if (pdet == (PANEL *) 0)
	LeaveText ("cannot make detail panel", 1);

    show_panel (pdet);
    top_panel (pdet);
    wdet = panel_window (pdet);

    IpDspInit (wdet, 0, 0);
    IpDisplay (wdet, 0, 0);
}

static void DetailIpUpdate ()
{
    IpDisplay (wdet, 0, 0);
}

/*
**
**  TCP support
**
*/
#define DETAIL_TCP_TLY         ((LINES > 38) ? (PER_SEC_TLY + 15) : PER_SEC_TLY)
#define DETAIL_TCP_LENGTH      (LINES - DETAIL_TCP_TLY)

static void DetailTcpInit ()
{
    pdet = MakePanel (DETAIL_TCP_LENGTH, COLS, DETAIL_TCP_TLY, 0, "tcp");
    if (pdet == (PANEL *) 0)
	LeaveText ("cannot make detail panel", 1);

    show_panel (pdet);
    top_panel (pdet);
    wdet = panel_window (pdet);

    TcpDspInit (wdet, 0, 0);
    TcpDisplay (wdet, 0, 0);
}

static void DetailTcpUpdate ()
{
    TcpDisplay (wdet, 0, 0);
}

/*
**
**  HARD support
**
*/
#define DETAIL_HARD_TLY         ((LINES > 38) ? (PER_SEC_TLY + 15) : PER_SEC_TLY)
#define DETAIL_HARD_LENGTH      (LINES - DETAIL_HARD_TLY)

static void DetailHardInit ()
{
    pdet = MakePanel (DETAIL_HARD_LENGTH, COLS, DETAIL_HARD_TLY, 0, "hard");
    if (pdet == (PANEL *) 0)
	LeaveText ("cannot make detail panel", 1);

    show_panel (pdet);
    top_panel (pdet);
    wdet = panel_window (pdet);

    DspHardLiterals (wdet, 0, 0);
    DspHard (wdet, 0, 0);
}

static void DetailHardUpdate ()
{
    DspHard (wdet, 0, 0);
}

/*
**
**  SERIAL support
**
*/
#define DETAIL_SERIAL_TLY          ((LINES > 38) ? (PER_SEC_TLY + 15) : PER_SEC_TLY)
#define DETAIL_SERIAL_LENGTH       (LINES - DETAIL_SERIAL_TLY)

static void DetailSerialInit ()
{
    pdet = MakePanel (DETAIL_SERIAL_LENGTH, COLS, DETAIL_SERIAL_TLY, 0, "serial");
    if (pdet == (PANEL *) 0)
	LeaveText ("cannot make detail panel", 1);

    show_panel (pdet);
    top_panel (pdet);
    wdet = panel_window (pdet);

#if defined(MODULE_FAS)
    DspFasLiterals (wdet, 0, 0);
    DspFas (wdet, 0, 0);
#elif defined(MODULE_SAS)
    DspSasLiterals (wdet, 0, 0);
    DspSas (wdet, 0, 0);
#elif defined(MODULE_ASY)
    DspAsyLiterals (wdet, 0, 0);
    DspAsy (wdet, 0, 0);
#else
    {
	char    *message = "Serial port statistics not compiled in, sorry!";
	int     x;

	use_cp (wdet, cpBANNER);
	wmove (wdet, 0, 0);
	for (x = 0; x < getmaxx (wdet); x++)
	    waddch (wdet, (chtype) '-');

	use_cp (wdet, cpBANWARN);
	mvwaddstr (wdet,
	    (getmaxy (wdet) - 1) / 2,
	    (getmaxx (wdet) - strlen (message)) / 2,
	    message
	);

	wclrtoeol (wdet);
	wclrtobot (wdet);
    }
#endif
}

static void DetailSerialUpdate ()
{
#if defined(MODULE_FAS)
    DspFas (wdet, 0, 0);
#elif defined(MODULE_SAS)
    DspSas (wdet, 0, 0);
#elif defined(MODULE_ASY)
    DspAsy (wdet, 0, 0);
#endif
}

/*
**
**  STREAMS support
**
*/
#define DETAIL_STREAMS_TLY      ((LINES > 38) ? (PER_SEC_TLY + 15) : PER_SEC_TLY)
#define DETAIL_STREAMS_LENGTH   (LINES - DETAIL_STREAMS_TLY)

static void DetailStreamInit ()
{
    pdet = MakePanel (DETAIL_STREAMS_LENGTH, COLS, DETAIL_STREAMS_TLY, 0, "str");
    if (pdet == (PANEL *) 0)
	LeaveText ("cannot make detail panel", 1);

    show_panel (pdet);
    top_panel (pdet);
    wdet = panel_window (pdet);

    DspStreamLiterals (wdet, 0, 0);
    DspStream (wdet, 0, 0);
}

static void DetailStreamUpdate ()
{
    DspStream (wdet, 0, 0);
}

/*
**
**  general DETAIL functions
**
*/
static void DetailPanelDestroy ()
{
    hide_panel (pdet);
    delwin (wdet);
    wdet = (WINDOW *) 0;
    del_panel (pdet);
    top_panel (pscr);
    detailPanelType = DPT_NONE;
}

void DetailCommand (chtype cmd)
{
    switch (cmd)
    {
	case 'm':
	    if (LINES <= 38 && detailPanelType != DPT_NONE)
		DetailPanelDestroy ();
	    else if (detailPanelType == DPT_PROC_LONG)
	    {
		DetailPanelDestroy ();
		DetailProcInit (0);
		detailPanelType = DPT_PROC;
	    }
	    break;

	case 'P':
	    if (LINES == 24)
	    {
		beep ();
		break;
	    }
	    if (detailPanelType == DPT_PROC_LONG)
		break;
	    if (detailPanelType != DPT_NONE)
		DetailPanelDestroy ();
	    DetailProcInit (1);
	    detailPanelType = DPT_PROC_LONG;
	    break;

	case 'p':
	    if (detailPanelType == DPT_PROC)
		break;
	    if (detailPanelType != DPT_NONE)
		DetailPanelDestroy ();
	    DetailProcInit (0);
	    detailPanelType = DPT_PROC;
	    break;

	case 'e':
	    if (detailPanelType == DPT_EXTRA)
		break;
	    if (detailPanelType != DPT_NONE)
		DetailPanelDestroy ();
	    DetailExtraInit ();
	    detailPanelType = DPT_EXTRA;
	    break;

	case 'h':
	    if (detailPanelType == DPT_HACK)
		break;
	    if (detailPanelType != DPT_NONE)
		DetailPanelDestroy ();
	    DetailHackInit ();
	    detailPanelType = DPT_HACK;
	    break;

	case 'c':
	    if (detailPanelType == DPT_SCSI)
		break;
	    if (detailPanelType != DPT_NONE)
		DetailPanelDestroy ();
	    DetailScsiInit ();
	    detailPanelType = DPT_SCSI;
	    break;

	case '?':
	case KEY_HELP:
	    if (detailPanelType == DPT_HELP)
		break;
	    if (detailPanelType != DPT_NONE)
		DetailPanelDestroy ();
	    DetailHelpInit ();
	    detailPanelType = DPT_HELP;
	    break;

	case 'z':
	    if (detailPanelType == DPT_SLIP)
		break;
	    if (detailPanelType != DPT_NONE)
		DetailPanelDestroy ();
	    DetailSlipInit ();
	    detailPanelType = DPT_SLIP;
	    break;

	case 'i':
	    if (detailPanelType == DPT_IP)
		break;
	    if (detailPanelType != DPT_NONE)
		DetailPanelDestroy ();
	    DetailIpInit ();
	    detailPanelType = DPT_IP;
	    break;

	case 't':
	    if (detailPanelType == DPT_TCP)
		break;
	    if (detailPanelType != DPT_NONE)
		DetailPanelDestroy ();
	    DetailTcpInit ();
	    detailPanelType = DPT_TCP;
	    break;

	case 'a':
	    if (detailPanelType == DPT_HARD)
		break;
	    if (detailPanelType != DPT_NONE)
		DetailPanelDestroy ();
	    DetailHardInit ();
	    detailPanelType = DPT_HARD;
	    break;

	case 'x':
	    if (detailPanelType == DPT_SERIAL)
		break;
	    if (detailPanelType != DPT_NONE)
		DetailPanelDestroy ();
	    DetailSerialInit ();
	    detailPanelType = DPT_SERIAL;
	    break;

	case 's':
	    if (detailPanelType == DPT_STREAM)
		break;
	    if (detailPanelType != DPT_NONE)
		DetailPanelDestroy ();
	    DetailStreamInit ();
	    detailPanelType = DPT_STREAM;
	    break;

	default:
	    if (detailPanelType == DPT_SERIAL)
	    {
#if defined(MODULE_FAS)
		FasCommand (cmd);
#elif defined(MODULE_SAS)
		SasCommand (cmd);
#elif defined(MODULE_ASY)
		AsyCommand (cmd);
#else
		beep ();
#endif
	    }
	    else if (detailPanelType == DPT_PROC
		|| detailPanelType == DPT_PROC_LONG
	    )
	    {
		ProcCommand (cmd);
	    }
	    else if (detailPanelType == DPT_SCSI)
	    {
		ScsiCommand (cmd);
	    }
	    else if (detailPanelType == DPT_HELP)
	    {
		HelpCommand (cmd);
	    }
	    else
		beep ();
	    break;
    }
}

void DspDetailPanel ()
{
    switch (detailPanelType)
    {
	case DPT_PROC:
	case DPT_PROC_LONG:
	    DetailProcUpdate ();
	    break;
	case DPT_EXTRA:
	    DetailExtraUpdate ();
	    break;
	case DPT_HACK:
	    DetailHackUpdate ();
	    break;
	case DPT_SCSI:
	    DetailScsiUpdate ();
	    break;
	case DPT_HELP:
	    DetailHelpUpdate ();
	    break;
	case DPT_SLIP:
	    DetailSlipUpdate ();
	    break;
	case DPT_IP:
	    DetailIpUpdate ();
	    break;
	case DPT_TCP:
	    DetailTcpUpdate ();
	    break;
	case DPT_HARD:
	    DetailHardUpdate ();
	    break;
	case DPT_SERIAL:
	    DetailSerialUpdate ();
	    break;
	case DPT_STREAM:
	    DetailStreamUpdate ();
	    break;
    }
}

void DetailInit ()
{
    BootInit ();
    HackInit ();
    HardInit ();
    HelpInit ();
    SlipInit ();
    IpVarInit ();
    ProcInit ();
    ScsiInit ();
    StreamInit ();
    TcpVarInit ();
    TuneInit ();
    VarInit ();

#if defined(MODULE_FAS)
    FasInit ();
#elif defined(MODULE_SAS)
    SasInit ();
#elif defined(MODULE_ASY)
    AsyInit ();
#endif
}


void DetailModeChange ()
{
    TcpModeChange ();
    IpModeChange ();
    SlipModeChange ();
}
