/*
**
**                                H A R D . C
**
**
**      Module:     hardware details display
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      13-OCT-1992 AV  Version X1.0, first definition
**      20-OCT-1992 AV  conditionalize for ARCH_X86
**      23-OCT-1992 AV  Version 1.0
**      06-NOV-1992 AV  conditionalize var's access, don't panic on missing vars
**      21-OCT-1993 AV  UNIVEL fixes
**
*/

#include "config.h"

#if defined(MODULE_HARD)

#include <stdlib.h>
#include <curses.h>
#include <panel.h>
#include <termios.h>

#include <libgen.h>
#include <fcntl.h>
#include <sys/fp.h>
#include <sys/types.h>
#include <sys/vtoc.h>
#include <sys/stat.h>
#include <sys/alttbl.h>
#if defined (S_UNIVEL)
#   include <sys/athd.h>
#else
#   include <sys/hd.h>
#endif
#include <sys/cram.h>
#include <sys/bootinfo.h>

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"

/*
**   00000000001111111111222222222233333333334444444444555555555566666666667777777777
**   01234567890123456789012345678901234567890123456789012345678901234567890123456789
**
**00 --------- Hardware configuration -----------------------------------------------
**01 ID    DISK       TYPE  #CYL SEC/TRK #PART #HEADS BYT/SEC  ALTS   CAPACITY
**02 ##### ###### ######## #####   ##### #####  #####   ##### ##### ####### MB
**03 ##### ###### ######## #####   ##### #####  #####   ##### ##### ####### MB
**04 ##### ###### ######## #####   ##### #####  #####   ##### ##### ####### MB
**05 ##### ###### ######## #####   ##### #####  #####   ##### ##### ####### MB
**06
**07 Floppy 0: ##.##MB ##.##"     Floppy 1: ##.##MB ##.##"
**08
**09 Main Processor: #########
**10 Math Processor: #########    do386b1 #  do386b1_387 #  do386b1_x87 #  do387cr3 #
**11
**12 Base Memory: ######k  Extended Memory: ######k  Expanded Memory: ######k
*/

#define ADDR_architecture   kmemSymbols[0].n_value
#define ADDR_fp_kind        kmemSymbols[1].n_value
#define ADDR_do386b1        kmemSymbols[2].n_value
#define ADDR_do386b1_387    kmemSymbols[3].n_value
#define ADDR_do386b1_x87    kmemSymbols[4].n_value
#define ADDR_do387cr3       kmemSymbols[5].n_value

static struct nlist kmemSymbols[] =
{
    { "architecture"        },
    { "fp_kind"             },
    { "do386b1"             },
    { "do386b1_387"         },
    { "do386b1_x87"         },
    { "do387cr3"            },
    { 0                     },
};

/*
**  Search list for hard disk drives. The first dimension specifies different
**  drives and the second different names for one drive. This structure
**  could be expanded if you find more drive names to search for.
*/
static char *hdTab[4][2] = {
    { "/dev/rdsk/0s0", "/dev/rdsk/c8d0s0" },
    { "/dev/rdsk/1s0", "/dev/rdsk/c8d1s0" },
    { "/dev/rdsk/2s0", "/dev/rdsk/c8d2s0" },
    { "/dev/rdsk/3s0", "/dev/rdsk/c8d3s0" },
};

#define CRAM_FILE   "/dev/cram"

typedef struct hdinfo
{
    int     id;
    int     type;
    int     cyl;
    int     tracks;
    int     sec;
    int     bytes;
    int     npart;
    int     alttbl_size;
    int     sizeMB;

} hdinfo_t;


static int GetHardDiskInfo (char *devname, hdinfo_t *hd)
{
    int                 dskfd;
    struct disk_parms   dp;
    char                *disk_buf;
    struct pdinfo       pdinfo;
    struct vtoc         vtoc;
    struct alt_info     alttbl;
    int                 valid = 1;

    if ((dskfd = open (devname, O_RDONLY)) == -1)
	return (0);

    if (ioctl (dskfd, V_GETPARMS, &dp) == -1)
    {
	(void) close (dskfd);
	return (0);
    }

    hd->sizeMB = ((long) (dp.dp_cyls
	* dp.dp_sectors * dp.dp_secsiz * dp.dp_heads)) >> 20;

    if ((disk_buf = (char *) malloc (sizeof (alttbl))) == NULL)
    {
	(void) close (dskfd);
	return (0);
    }

    if (lseek (dskfd, dp.dp_secsiz * VTOC_SEC, 0) == -1
	|| read (dskfd, disk_buf, dp.dp_secsiz) == -1
    )
    {
	free (disk_buf);
	(void) close (dskfd);
	return (0);
    }

    (void) memcpy ((char *) &pdinfo, disk_buf, sizeof (pdinfo));

#ifdef S_UNIVEL
    if (pdinfo.sanity != VALID_PD || pdinfo.version < V_VERSION_1
	|| pdinfo.version > V_VERSION
    )
#else
    if (pdinfo.sanity != VALID_PD || pdinfo.version != V_VERSION)
#endif
	valid = 0;
    else
    {
	(void) memcpy ((char *) &vtoc, &disk_buf[pdinfo.vtoc_ptr % dp.dp_secsiz],
	    sizeof (vtoc)
	);

#ifdef S_UNIVEL
	if (vtoc.v_sanity != VTOC_SANE || vtoc.v_version < V_VERSION_1
	    || vtoc.v_version > V_VERSION
	)
#else
	if (vtoc.v_sanity != VTOC_SANE || vtoc.v_version != V_VERSION)
#endif
	    valid = 0;
    }

    hd->type   = dp.dp_type;
    hd->cyl    = dp.dp_cyls;
    hd->tracks = dp.dp_heads;
    hd->sec    = dp.dp_sectors;
    hd->bytes  = dp.dp_secsiz;

    if (valid)
    {
	hd->id          = pdinfo.driveid;
	hd->npart       = vtoc.v_nparts;
	hd->alttbl_size = pdinfo.alt_len;
    }
    else
    {
	hd->id          =
	hd->npart       =
	hd->alttbl_size = -1;
    }

    free (disk_buf);
    close (dskfd);
    return (1);
}

static char *GetFloppyData (int fdtype)
{
    static char *typeStr[] = {
	"        UNKNOWN",
	"  360KB  5.25\"",
	"  1.2MB  5.25\"",
	"  720KB   3.5\"",
	" 1.44MB   3.5\""
    };

    if (fdtype < 0 || fdtype > sizeof (typeStr))
	fdtype = 0;

    return (typeStr[fdtype]);
}

DspHardLiterals (WINDOW *win, int ty, int tx)
{
    static char     *header = "--------- Hardware configuration ";
    int             cramfd;
    unsigned char   crambuf[4];
    hdinfo_t        hdinf;
    int             fpkind;
    int             idx;
    int             y;
    int             x;
    int             didx;
    int             aidx;

    wmove (win, ty, tx);

    use_cp (win, cpBANNER);
    waddstr (win, header);
    getyx (win, y, x);
    while (x++ < getmaxx (win))
	waddch (win, (chtype) '-');

    use_cp (win, cpLIT);

    mvwaddstr (win, ty +  1, tx +  0, "ID    DISK       TYPE  #CYL SEC/TRK #PART #HEADS BYT/SEC  ALTS   CAPACITY");
    y = ty + 2;
    idx = 0;

    for (didx = 0; didx < sizeof (hdTab) / sizeof (hdTab[0]); didx++)
    {
	if (idx >= 4)
	    break;

	if (hdTab[didx] == NULL)
	    continue;

	for (aidx = 0; aidx < sizeof (*hdTab) / sizeof (hdTab[0][0]); aidx++)
	{
	    if (hdTab[didx][aidx] == NULL)
		continue;

	    if (GetHardDiskInfo (hdTab[didx][aidx], &hdinf))
	    {
		if (hdinf.id == -1)
		    mvwaddstr (win, y, tx + 0, "??   ");
		else
		    mvwprintw (win, y, tx + 0, "%-5d", hdinf.id);

		mvwprintw (win, y, tx + 6, "%-6s", basename (hdTab[didx][aidx]));

		if (hdinf.type == DPT_WINI)
		    mvwprintw (win, y, tx + 13, "WINCHEST");
		else if (hdinf.type == DPT_FLOPPY)
		    mvwprintw (win, y, tx + 13, "  FLOPPY");
		else if (hdinf.type == DPT_OTHER)
		    mvwprintw (win, y, tx + 13, "   OTHER");
		else if (hdinf.type == DPT_NOTDISK)
		    mvwprintw (win, y, tx + 13, " NOTDISK");
		else if (hdinf.type == DPT_SCSI_HD)
		    mvwprintw (win, y, tx + 13, " SCSI_HD");
		else if (hdinf.type == DPT_SCSI_OD)
		    mvwprintw (win, y, tx + 13, " SCSI_OD");
		else if (hdinf.type == DPT_SCSI_FD)
		    mvwprintw (win, y, tx + 13, " SCSI_FD");
		else if (hdinf.type == DPT_ESDI_HD)
		    mvwprintw (win, y, tx + 13, " ESDI_HD");
		else
		    mvwprintw (win, y, tx + 13, " UNKNOWN");

		mvwprintw (win, y, tx + 22, "%5d",    hdinf.cyl);
		mvwprintw (win, y, tx + 30, "%5d",    hdinf.sec);

		if (hdinf.npart == -1)
		    mvwaddstr (win, y, tx + 36, "   ??");
		else
		    mvwprintw (win, y, tx + 36, "%5d", hdinf.npart);

		mvwprintw (win, y, tx + 43, "%5d",    hdinf.tracks);
		mvwprintw (win, y, tx + 51, "%5d",    hdinf.bytes);

		if (hdinf.alttbl_size == -1)
		    mvwaddstr (win, y, tx + 57, "   ??");
		else
		    mvwprintw (win, y, tx + 57, "%5d", hdinf.alttbl_size);

		mvwprintw (win, y, tx + 63, "%7d MB", hdinf.sizeMB);

		idx++;
		y++;
		break;
	    }
	}
    }

    if ((cramfd = open (CRAM_FILE, O_RDONLY)) == -1)
	return;

    mvwaddstr (win, ty +  7, tx +  0, "Floppy 0:");
    mvwaddstr (win, ty +  7, tx + 29, "Floppy 1:");
    crambuf[0] = DDTB;
    if (ioctl (cramfd, CMOSREAD, crambuf) == -1)
    {
	mvwaddstr (win, ty +  7, tx + 10, GetFloppyData (-1));
	mvwaddstr (win, ty +  7, tx + 39, GetFloppyData (-1));
    }
    else
    {
	mvwaddstr (win, ty +  7, tx + 10, GetFloppyData ((crambuf[1] >> 4) & 0x0f));
	mvwaddstr (win, ty +  7, tx + 39, GetFloppyData ((crambuf[1] >> 0) & 0x0f));
    }

    mvwaddstr (win, ty + 12, tx +  0, "Base Memory:");
    crambuf[0] = BMLOW;
    crambuf[2] = BMHIGH;
    if (ioctl (cramfd, CMOSREAD, &crambuf[0]) != -1
	&& ioctl (cramfd, CMOSREAD, &crambuf[2]) != -1
    )
	mvwprintw (win, ty + 12, tx + 13, "%6dk", (int) (crambuf[3] << 8) | crambuf[1]);
    else
	mvwaddstr (win, ty + 12, tx + 13, "UNKNOWN");

    mvwaddstr (win, ty + 12, tx + 22, "Extended Memory:");
    crambuf[0] = EMLOW;
    crambuf[2] = EMHIGH;
    if (ioctl (cramfd, CMOSREAD, &crambuf[0]) != -1
	&& ioctl (cramfd, CMOSREAD, &crambuf[2]) != -1
    )
	mvwprintw (win, ty + 12, tx + 39, "%6dk", (int) (crambuf[3] << 8) | crambuf[1]);
    else
	mvwaddstr (win, ty + 12, tx + 39, "UNKNOWN");

    mvwaddstr (win, ty + 12, tx + 48, "Expanded Memory:");
    crambuf[0] = EMLOW2;
    crambuf[2] = EMHIGH2;
    if (ioctl (cramfd, CMOSREAD, &crambuf[0]) != -1
	&& ioctl (cramfd, CMOSREAD, &crambuf[2]) != -1
    )
	mvwprintw (win, ty + 12, tx + 65, "%6dk", (int) (crambuf[3] << 8) | crambuf[1]);
    else
	mvwaddstr (win, ty + 12, tx + 65, "UNKNOWN");

    (void) close (cramfd);

    mvwaddstr (win, ty +  9, tx +   0, "Main Processor:");
    if (ADDR_architecture != 0)
	mvwprintw (win, ty +  9, tx +  16, "%9s", KTUT (ADDR_architecture, char *));
    else
	mvwprintw (win, ty +  9, tx +  16, "%9s", "Unknown");

    mvwaddstr (win, ty + 10, tx +  0, "Math Processor:");
    if (ADDR_fp_kind != 0)
	fpkind = (int) *KTUT (ADDR_fp_kind, char *);
    else
	fpkind = 0;

    if (fpkind == 0)
	mvwaddstr (win, ty + 10, tx + 16, "    NO-FP");
    else if (fpkind == 1)
	mvwaddstr (win, ty + 10, tx + 16, "SOFT-EMUL");
    else if (fpkind == 2)
	mvwaddstr (win, ty + 10, tx + 16, "   i80287");
    else if (fpkind == 3)
	mvwaddstr (win, ty + 10, tx + 16, "   i80387");
    else
	mvwprintw (win, ty + 10, tx + 16, "%6u ??", fpkind);

    if (ADDR_do386b1 != 0)
    {
	mvwaddstr (win, ty + 10, tx + 28, "do386b1:");
	mvwprintw (win, ty + 10, tx + 37, "%1d", *KTUT (ADDR_do386b1,     int *));
    }

    if (ADDR_do386b1_387 != 0)
    {
	mvwaddstr (win, ty + 10, tx + 39, "do386b1_387:");
	mvwprintw (win, ty + 10, tx + 52, "%1d", *KTUT (ADDR_do386b1_387, int *));
    }

    if (ADDR_do386b1_x87 != 0)
    {
	mvwaddstr (win, ty + 10, tx + 54, "do386b1_x87:");
	mvwprintw (win, ty + 10, tx + 67, "%1d", *KTUT (ADDR_do386b1_x87, int *));
    }

    if (ADDR_do387cr3 != 0)
    {
	mvwaddstr (win, ty + 10, tx + 69, "do387cr3:");
	mvwprintw (win, ty + 10, tx + 79, "%1d", *KTUT (ADDR_do387cr3,    int *));
    }
}


void DspHard (WINDOW *win, int ty, int tx)
{
    int     y;
    int     x;

    use_cp (win, cpINFO);

    wmove (win, getmaxy (win) - 1, getmaxx (win) - 1);
}

void HardInit ()
{
    (void) GetSymbols (kmemSymbols, 0);
}

#else

#include <curses.h>
#include <panel.h>

#include "svr4mon.h"
#include "dsputil.h"

void DspHardLiterals (WINDOW *win, int tly, int tlx)
{
    char    *message = "Architecture info not compiled in, sorry!";
    int     x;

    use_cp (win, cpBANNER);
    wmove (win, 0, 0);
    for (x = 0; x < getmaxx (win); x++)
	waddch (win, (chtype) '-');

    use_cp (win, cpBANWARN);
    mvwaddstr (win,
	(getmaxy (win) - 1) / 2,
	(getmaxx (win) - strlen (message)) / 2,
	message
    );

    wclrtoeol (win);
    wclrtobot (win);
}

void DspHard (WINDOW *win, int tly, int tlx)
{
}

void HardInit ()
{
}

#endif
