/*
**
**                                  I P . C
**
**
**      Module:     ip, icmp, udp statistics
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      16-OCT-1992 AV  Version X1.0, first definition
**      16-OCT-1992 AV  display changed values in "bold" mode
**      23-OCT-1992 AV  Version 1.0
**      28-OCT-1992 AV  fixed bug in updating title
**      11-NOV-1993 AV  added MIPS support and LOGGING support
**      14-NOV-1993 AV  split display function in calc in display function
*/

#include "config.h"

#if defined(MODULE_IP)

#include <stdlib.h>
#include <curses.h>
#include <panel.h>

#include "ip.h"

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"

/*
**  00000000001111111111222222222233333333334444444444555555555566666666667777777777
**  01234567890123456789012345678901234567890123456789012345678901234567890123456789
**  -------- IP ----------- - ICMP -------------------------------- ---- UDP -------
**  total        ########## error    ########## badcode  ########## hdrops   #######
**  badsum       ########## oldshort ########## tooshort ########## badsum   #######
**  tooshort     ########## oldicmp  ########## checksum ########## badlen   #######
**  toosmall     ########## reflect  ########## badlen   ########## fullsock #######
**  badhlen      ########## Output histogram:   Input histogram:
**  badlen       ########## echo reply ######## echo reply ########
**  fragments    ########## dest unrch ######## dest unrch ########
**  fragdropped  ########## echo       ######## echo       ########
**  fragtimeout  ########## time exeed ######## time exeed ########
**  forward      ########## redirect   ######## redirect   ########
**  cantforward  ########## iphead bad ######## iphead bad ########
**  redirectsent ########## pckt lost  ######## pckt lost  ########
**                          inf reqst  ######## inf reqst  ########
*/

#define ADDR_icmpstat       kmemSymbols[0].n_value
#define ADDR_ipstat         kmemSymbols[1].n_value
#define ADDR_udpstat        kmemSymbols[2].n_value

static struct nlist kmemSymbols[] =
{
#if defined(S_SINIX_MIPS)
    { "icmpstat_s"          },
    { "ipstat_s"            },
    { "udpstat_s"           },
#else
    { "icmpstat"            },
    { "ipstat"              },
    { "udpstat"             },
#endif
    { 0                     }
};


static struct ipstat    curIp;
static struct ipstat    oldIp;
struct ipstat           outIp;

static struct icmpstat  curIcmp;
static struct icmpstat  oldIcmp;
struct icmpstat         outIcmp;

static struct udpstat   curUdp;
static struct udpstat   oldUdp;
struct udpstat          outUdp;

static int              noSyms;

#define TSI     0   /* type signed int */
#define TUL     1   /* type unsigned long */

#define CURVAL(ptr) \
    (ptr->type == TUL ? *((u_long *) ptr->cp) : *((int *) ptr->cp))

#define OLDVAL(ptr) \
    (ptr->type == TUL ? *((u_long *) ptr->op) : *((int *) ptr->op))

#define OUTVAL(ptr) \
    (ptr->type == TUL ? *((u_long *) ptr->outp) : *((int *) ptr->outp))

#define SET_OUTVAL(ptr, val) \
    (ptr->type == TUL ? (*((u_long *) ptr->outp) = (val)) : (*((int *) ptr->outp) = (val)))

#define DefIpTag(name)      &curIp.name,   &oldIp.name,   &outIp.name
#define DefUdpTag(name)     &curUdp.name,  &oldUdp.name,  &outUdp.name
#define DefIcmpTag(name)    &curIcmp.name, &oldIcmp.name, &outIcmp.name
#define NullTag()           0, 0, 0

static struct descrip {
    int         y, tx, vx;
    char        *label;
    char        *fmt;
    int         type;
    void        *cp;
    void        *op;
    void        *outp;
} desc[] = {

#if defined(NETWORK_WOLLONGONG) && defined(SNMP)
    {  1,  0, 13, "InReceives",        "%10ld", TUL, DefIpTag (ipInReceives) },
#elif defined(NETWORK_LACHMAN)
    {  1,  0, 13, "total",             "%10ld", TUL, DefIpTag (ips_total) },
#endif

    {  2,  0, 13, "badsum",            "%10ld", TUL, DefIpTag (ips_badsum) },
    {  3,  0, 13, "tooshort",          "%10ld", TUL, DefIpTag (ips_tooshort) },
    {  4,  0, 13, "toosmall",          "%10ld", TUL, DefIpTag (ips_toosmall) },
    {  5,  0, 13, "badhlen",           "%10ld", TUL, DefIpTag (ips_badhlen) },
    {  6,  0, 13, "badlen",            "%10ld", TUL, DefIpTag (ips_badlen) },

#if defined (NETWORK_WOLLONGONG)
    {  7,  0, 13, "badttl",            "%10ld", TUL, DefIpTag (ips_badttl) },
#   if defined(SNMP)
    {  8,  0, 13, "OutRequests",       "%10ld", TUL, DefIpTag (ipOutRequests) },
    {  9,  0, 13, "OutDiscards",       "%10ld", TUL, DefIpTag (ipOutDiscards) },
#   endif
#elif defined (NETWORK_LACHMAN)
    {  7,  0, 13, "fragments",         "%10ld", TUL, DefIpTag (ips_fragments) },
    {  8,  0, 13, "fragdropped",       "%10ld", TUL, DefIpTag (ips_fragdropped) },
    {  9,  0, 13, "fragtimeout",       "%10ld", TUL, DefIpTag (ips_fragtimeout) },
#endif

    { 10,  0, 13, "forward",           "%10ld", TUL, DefIpTag (ips_forward) },
    { 11,  0, 13, "cantforward",       "%10ld", TUL, DefIpTag (ips_cantforward) },
    { 12,  0, 13, "redirectsent",      "%10ld", TUL, DefIpTag (ips_redirectsent) },

    {  1, 24, 33, "error",             "%10d",  TSI, DefIcmpTag (icps_error) },
    {  2, 24, 33, "oldshort",          "%10d",  TSI, DefIcmpTag (icps_oldshort) },
    {  3, 24, 33, "oldicmp",           "%10d",  TSI, DefIcmpTag (icps_oldicmp) },
    {  4, 24, 33, "reflect",           "%10d",  TSI, DefIcmpTag (icps_reflect) },
    {  5, 24, 33, "Output histogram:", "",      TSI, NullTag () },
    {  6, 24, 35, "echo reply",        "%8d",   TSI, DefIcmpTag (icps_outhist[ICMP_ECHOREPLY]) },
    {  7, 24, 35, "dest unrch",        "%8d",   TSI, DefIcmpTag (icps_outhist[ICMP_UNREACH]) },
    {  8, 24, 35, "echo",              "%8d",   TSI, DefIcmpTag (icps_outhist[ICMP_ECHO]) },
    {  9, 24, 35, "time exeed",        "%8d",   TSI, DefIcmpTag (icps_outhist[ICMP_TIMXCEED]) },
    { 10, 24, 35, "redirect",          "%8d",   TSI, DefIcmpTag (icps_outhist[ICMP_REDIRECT]) },
    { 11, 24, 35, "iphead bad",        "%8d",   TSI, DefIcmpTag (icps_outhist[ICMP_PARAMPROB]) },
    { 12, 24, 35, "pckt lost",         "%8d",   TSI, DefIcmpTag (icps_outhist[ICMP_SOURCEQUENCH]) },
    { 13, 24, 35, "inf reqst",         "%8d",   TSI, DefIcmpTag (icps_outhist[ICMP_IREQ]) },

    {  1, 44, 53, "badcode",           "%10d",  TSI, DefIcmpTag (icps_badcode) },
    {  2, 44, 53, "tooshort",          "%10d",  TSI, DefIcmpTag (icps_tooshort) },
    {  3, 44, 53, "checksum",          "%10d",  TSI, DefIcmpTag (icps_checksum) },
    {  4, 44, 53, "badlen",            "%10d",  TSI, DefIcmpTag (icps_badlen) },
    {  5, 44, 53, "Input histogram:",  "",      TSI, NullTag () },
    {  6, 44, 55, "echo reply",        "%8d",   TSI, DefIcmpTag (icps_inhist[ICMP_ECHOREPLY]) },
    {  7, 44, 55, "dest unrch",        "%8d",   TSI, DefIcmpTag (icps_inhist[ICMP_UNREACH]) },
    {  8, 44, 55, "echo",              "%8d",   TSI, DefIcmpTag (icps_inhist[ICMP_ECHO]) },
    {  9, 44, 55, "time exeed",        "%8d",   TSI, DefIcmpTag (icps_inhist[ICMP_TIMXCEED]) },
    { 10, 44, 55, "redirect",          "%8d",   TSI, DefIcmpTag (icps_inhist[ICMP_REDIRECT]) },
    { 11, 44, 55, "iphead bad",        "%8d",   TSI, DefIcmpTag (icps_inhist[ICMP_PARAMPROB]) },
    { 12, 44, 55, "pckt lost",         "%8d",   TSI, DefIcmpTag (icps_inhist[ICMP_SOURCEQUENCH]) },
    { 13, 44, 55, "inf reqst",         "%8d",   TSI, DefIcmpTag (icps_inhist[ICMP_IREQ]) },

    {  1, 64, 73, "hdrops",            "%7d",   TSI, DefUdpTag (udps_hdrops) },
    {  2, 64, 73, "badsum",            "%7d",   TSI, DefUdpTag (udps_badsum) },
    {  3, 64, 73, "badlen",            "%7d",   TSI, DefUdpTag (udps_badlen) },

#if defined(NETWORK_LACHMAN)
    {  4, 64, 73, "fullsock",          "%7d",   TSI, DefUdpTag (udps_fullsock) },
#elif defined(NETWORK_WOLLONGONG)
    {  4, 64, 73, "qfull",             "%7d",   TSI, DefUdpTag (udps_qfull) },
    {  5, 64, 73, "busy",              "%7d",   TSI, DefUdpTag (udps_busy) },
#   if defined(SNMP)
    {  6, 64, 73, "InDatagram",        "%7d",   TSI, DefUdpTag (udpInDatagrams) },
    {  7, 64, 73, "OutDatagram",       "%7d",   TSI, DefUdpTag (udpOutDatagrams) },
    {  8, 64, 73, "NoPorts",           "%7d",   TSI, DefUdpTag (udpNoPorts) },
    {  9, 64, 73, "InErrors",          "%7d",   TSI, DefUdpTag (udpInErrors) },
#   endif
#endif
};


void IpVarInit (void)
{
    if ((noSyms = GetSymbols (kmemSymbols, 0)))
	return;

    if (infoMode != modeAbsolute)
    {
	(void) memcpy (&curIp,
	    KTUT (ADDR_ipstat, struct ipstat *), sizeof (struct ipstat)
	);
	(void) memcpy (&curIcmp,
	    KTUT (ADDR_icmpstat, struct icmpstat *), sizeof (struct icmpstat)
	);
	(void) memcpy (&curUdp,
	    KTUT (ADDR_udpstat, struct udpstat *), sizeof (struct udpstat)
	);
	(void) memcpy (&oldIp,   &curIp,   sizeof (struct ipstat));
	(void) memcpy (&oldIcmp, &curIcmp, sizeof (struct icmpstat));
	(void) memcpy (&oldUdp,  &curUdp,  sizeof (struct udpstat));
    }
}


void IpDspInit (WINDOW *win, int tly, int tlx)
{
    static char     *header = "-------- IP ----------- - ICMP -------------------------------- ---- UDP ";
    int             x;
    int             y;
    struct descrip  *dptr;

    wmove (win, tly, tlx);

    use_cp (win, cpBANNER);
    waddstr (win, header);
    getyx (win, y, x);
    while (x++ < getmaxx (win))
	waddch (win, (chtype) '-');

    if (noSyms)
    {
	char    *message = "IP related kernel variables not found!";
	int     x;

	use_cp (win, cpBANWARN);
	mvwaddstr (win,
	    (getmaxy (win) - 1) / 2,
	    (getmaxx (win) - strlen (message)) / 2,
	    message
	);

	wclrtoeol (win);
	wclrtobot (win);
	return;
    }

    use_cp (win, cpLIT);

    for (x = 0, dptr = desc; x < sizeof (desc) / sizeof (desc[0]); dptr++, x++)
	mvwaddstr (win, tly + dptr->y, tlx + dptr->tx, dptr->label);
}


void IpPreCalc (void)
{
    struct descrip  *dptr;
    int             i;

    if (noSyms)
	return;

    (void) memcpy (&curIp,
	KTUT (ADDR_ipstat, struct ipstat *), sizeof (struct ipstat)
    );
    (void) memcpy (&curIcmp,
	KTUT (ADDR_icmpstat, struct icmpstat *), sizeof (struct icmpstat)
    );
    (void) memcpy (&curUdp,
	KTUT (ADDR_udpstat, struct udpstat *), sizeof (struct udpstat)
    );

    for (i = 0, dptr = desc; i < sizeof (desc) / sizeof (desc[0]); dptr++, i++)
    {
	if (dptr->fmt == 0 || dptr->fmt[0] == 0)
	    continue;

	if (infoMode == modeAbsolute)
	    SET_OUTVAL (dptr, CURVAL(dptr));
	else
	    SET_OUTVAL (dptr, CURVAL(dptr) - OLDVAL(dptr));
    }
}


void IpDisplay (WINDOW *win, int tly, int tlx)
{
    struct descrip  *dptr;
    int             i;
    int             flag;

    if (noSyms)
	return;

    DispTimer  (win, 0, 32, 63);

    use_cp (win, cpINFO);

    for (i = 0, dptr = desc; i < sizeof (desc) / sizeof (desc[0]); dptr++, i++)
    {
	if (dptr->fmt == 0 || dptr->fmt[0] == 0)
	    continue;

	if (infoMode == modeAbsolute)
	    mvwprintw (
		win, tly + dptr->y, tlx + dptr->vx, dptr->fmt, OUTVAL(dptr)
	    );
	else
	{
	    if ((flag = (CURVAL(dptr) != OLDVAL(dptr))))
		use_cp (win, cpHIGH);
	    mvwprintw (
		win, tly + dptr->y, tlx + dptr->vx, dptr->fmt, OUTVAL(dptr)
	    );
	    if (flag)
		use_cp (win, cpINFO);
	}
    }

    wmove (win, getmaxy (win) - 1, getmaxx (win) - 1);
}


void IpPostCalc (void)
{
    if (noSyms)
	return;

    if (infoMode == modeDelta)
    {
	(void) memcpy (&oldIp,   &curIp,   sizeof (struct ipstat));
	(void) memcpy (&oldIcmp, &curIcmp, sizeof (struct icmpstat));
	(void) memcpy (&oldUdp,  &curUdp,  sizeof (struct udpstat));
    }
}


void IpModeChange (void)
{
    if (noSyms)
    {
	beep ();
	return;
    }

    switch (infoMode)
    {
	case modeDelta:
	case modeRelative:
	    (void) memcpy (&oldIp,   &curIp,   sizeof (struct ipstat));
	    (void) memcpy (&oldIcmp, &curIcmp, sizeof (struct icmpstat));
	    (void) memcpy (&oldUdp,  &curUdp,  sizeof (struct udpstat));
	    break;
	case modeAbsolute:
	    break;
    }
}


void IpExit (void)
{
}


#else

#include <curses.h>
#include <panel.h>

#include "svr4mon.h"
#include "dsputil.h"

void IpVarInit (void) {}

void IpDspInit (WINDOW *win, int tly, int tlx)
{
    char    *message = "IP statistics not compiled in, sorry!";
    int     x;

    use_cp (win, cpBANNER);
    wmove (win, 0, 0);
    for (x = 0; x < getmaxx (win); x++)
	waddch (win, (chtype) '-');

    use_cp (win, cpBANWARN);
    mvwaddstr (win,
	(getmaxy (win) - 1) / 2,
	(getmaxx (win) - strlen (message)) / 2,
	message
    );

    wclrtoeol (win);
    wclrtobot (win);
}

void IpPreCalc (void) {}
void IpDisplay (WINDOW *win, int tly, int tlx) {}
void IpPostCalc (void) {}
void IpModeChange (void) {}
void IpExit (void) {}

#endif
