/*
**
**                            L O G G I N G . C
**
**  This module handles logging.
**
**      Module:     logging handling
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      11-NOV-1993 AV      Version X1.0, first definition
**
*/

#include <stdio.h>
#include <errno.h>
#include <malloc.h>

#include "logging.h"
#include "loghash.h"
#include "svr4mon.h"

static FILE 	*log_fp;


kvar_symbol_t *kvars_config_init (
    const char  *log_filename,
    const char  *conf_filename
)
{
    unsigned int    kvar_symbol_size  = 100;
    unsigned int    kvar_symbol_index = 0;
    kvar_symbol_t   *kvar_symbols;
    FILE            *fp;
    char            line[1024];

    /*
    **  Allocate an arry to hold the kvar information.
    */
    if ((kvar_symbols = (kvar_symbol_t *) malloc (sizeof (kvar_symbol_t) * kvar_symbol_size)) == (kvar_symbol_t *) NULL)
    {
	perror ("kvars_config_init: malloc");
	return ((kvar_symbol_t *) NULL);
    }

    kvar_symbol_index = 0;

    /*
    **  Initialize the new array.
    */
    memset (kvar_symbols, 0, sizeof (kvar_symbol_t) * kvar_symbol_size);

    /*
    **  Open the configuration file.
    */
    if (conf_filename == 0 || (fp = fopen (conf_filename, "r")) == (FILE *) NULL)
    {
	perror ("kvars_config_init: fopen");
	return ((kvar_symbol_t *) NULL);
    }

    /*
    **  Continue until the end-of-file.  fgets() will return NULL at
    **  end-of-file.
    */
    while (fgets (line, sizeof (line), fp) != (char *) NULL)
    {
	char                    kvar_symbol_name[1024];
	const kvar_symbol_t     *kvar_symbol;

	if (sscanf (line, "%s", kvar_symbol_name) != 1)
	{
	    /*
	    **  There was an error in processing the line.  Ignore it and
	    **  continue.
	    */
	    fprintf (stderr, "Ignoring unknown line syntax:\n%s\n", line);
	    continue;
	}

	if ((kvar_symbol = in_word_set (kvar_symbol_name, strlen (kvar_symbol_name))) == (kvar_symbol_t *) NULL)
	{
	    /*
	    **  The kernel symbol name was unknown.  Print an error message
	    **  and continue.
	    */
	    fprintf (stderr, "Unknown symbol name: %s\n", kvar_symbol_name);
	    continue;
	}

	kvar_symbols[kvar_symbol_index].name      = kvar_symbol->name;
	kvar_symbols[kvar_symbol_index].desc      = kvar_symbol->desc;
	kvar_symbols[kvar_symbol_index].value_ptr = kvar_symbol->value_ptr;
	kvar_symbol_index++;

	/*
	**  See if we have reached the end of the kvar_symbols array.
	**  If so allocate a bigger one.
	*/
	if (kvar_symbol_index == kvar_symbol_size)
	{
	    kvar_symbol_size *= 2;

	    /*
	    **  reallocate an arry to hold the kvar information.
	    */
	    if ((kvar_symbols = (kvar_symbol_t *) realloc (kvar_symbols, sizeof (kvar_symbol_t) * kvar_symbol_size)) == (kvar_symbol_t *) NULL)
	    {
		perror ("kvars_config_init: realloc");
		return ((kvar_symbol_t *) NULL);
	    }

	    /*
	    **  Initialize the new part of the array.
	    */
	    memset (&kvar_symbols[kvar_symbol_index], 0, sizeof (kvar_symbol_t) * (kvar_symbol_size / 2));
	}
    }

    (void) fclose(fp);

    /*
    **  Now we have the list of kernel variables which are going to
    **  be logged.  Open the log file and write out the header.
    */
    if (log_filename == 0 || (log_fp = fopen (log_filename, "w")) == (FILE *) NULL)
    {
	char    buffer[1024];

	(void) fprintf (stderr, "Cannot duplicate log filename: %s\n", log_filename);
	exit (1);
    }

    /*
    **  Print out the current date/time.
    */
    {
	time_t      now;
	struct tm   *lnow;

	now  = time (NULL);
	lnow = localtime (&now);
	(void) fprintf (log_fp, "svr4mon started: %s\n", asctime (lnow));
    }

    (void) fputs ("Symbols selected:\n-----------------\n", log_fp);
    {
	kvar_symbol_t   *kvar;

	for (kvar = kvar_symbols; kvar->name != (char *) NULL; kvar++)
	    (void) fprintf (log_fp, "%-20s - %s\n", kvar->name, kvar->desc);
    }
    (void) fputs ("\n", log_fp);

    /*
    **  Print out the header line.
    */
    (void) fprintf (log_fp, "%10s", "time");
    {
	kvar_symbol_t   *kvar;

	for (kvar = kvar_symbols; kvar->name != (char *) NULL; kvar++)
	    (void) fprintf (log_fp, "%20s", kvar->name);
    }
    (void) fputs ("\n", log_fp);

    return (kvar_symbols);
}


void kvar_log (struct tm *ltime, kvar_symbol_t *kvar_symbols)
{
    static int              firstTime = 1;
    static enum info_mode_e curMode;
    kvar_symbol_t           *kvar;

    if (log_fp == (FILE *) 0 || kvar_symbols == (kvar_symbol_t *) 0)
	return;

    if (firstTime || curMode != infoMode)
    {
	if (infoMode == modeDelta)
	    (void) fprintf (log_fp, "Mode: Delta\n");
	else if (infoMode == modeAbsolute)
	    (void) fprintf (log_fp, "Mode: Absolute\n");
	else if (infoMode == modeRelative)
	    (void) fprintf (log_fp, "Mode: Relative\n");
	else
	    /* cannot happen */;

	firstTime = 0;
	curMode = infoMode;
    }

    /*
    **  Print out the time; make sure that the time field takes up ten spaces
    */
    (void) fprintf (log_fp, "  %02d:%02d:%02d",
	ltime->tm_hour, ltime->tm_min, ltime->tm_sec
    );

    for (kvar = kvar_symbols; kvar->name != (char *) NULL; kvar++)
	(void) fprintf (log_fp, "%20d", *kvar->value_ptr);

    (void) fputs ("\n", log_fp);
}


void kvar_suspend (void)
{
    time_t now = time (NULL);
    (void) fprintf (log_fp, "Logging suspended: %s", asctime (localtime (&now)));
}


void kvar_resume (void)
{
    time_t now = time (NULL);
    (void) fprintf (log_fp, "Logging resumed:   %s", asctime (localtime (&now)));
}


void kvar_log_end (void)
{
    if (log_fp != (FILE *) 0)
	(void) fclose (log_fp);
}
