/*
**
**                              P A G I N G . C
**
**
**      Module:     demand paging utilities and symbols
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      09-OCT-1992 AV  Version X1.0, first definition
**      15-OCT-1992 AV  fas and sas entries were not correct
**      16-OCT-1992 AV  on SIGSEGV, check if requested address is really
**                      a kernel address, otherwise generate fault
**      17-OCT-1992 AV  use SYM_DUMMY for unavailble kernel vars
**                      pad fas and sas structures
**      20-OCT-1992 AV  asy driver vars (may vary from one SVR4 to another)
**      21-OCT-1992 AV  the kernel symbol table was splitted and
**                      distributed to the various modules. this makes
**                      loading of symbols somewhat slower, but now it's
**                      easier to port the program to other flavours of
**                      SVR4
**      21-OCT-1992 AV  start port for ARCH_MOT_88k
**      23-OCT-1992 AV  Version 1.0
**      30-OCT-1992 AV  let paramter to GetSymbols decide if we should
**                      check for the nlist() result
**      21-OCT-1993 AV  added support for Univel
**      11-NOV-1993 AV  added MIPS support and LOGGING support
**
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <nlist.h>

#include <sys/resource.h>
#include <sys/param.h>
#include <sys/mman.h>

#if defined(S_MOTOROLA_88K)
#   include <sys/sbd.h>
#elif defined(S_SINIX_MIPS)
#   include <sys/kmacct.h>
#else
#   include <sys/immu.h>
#endif

#include <sys/sysmacros.h>

#include <signal.h>
#include <sys/siginfo.h>
#include <sys/ucontext.h>

/*
**  damn - name space pollution
*/
#undef ERR

#include "svr4mon.h"
#include "dsputil.h"
#include "paging.h"

caddr_t     pagingBase = (caddr_t) 0;
caddr_t     minkvadr = 0;
caddr_t     maxkvadr = 0;

static int  kmem_fd = -1;

int GetSymbols (struct nlist *sptr, int check)
{
    int     failed = 0;

    /*
    **  fetch kernel symbols
    */
    if (nlist (VMUNIX_FILE, sptr) == -1)
	LeaveText ("nlist", 255);

    /*
    **  check for errors
    */
    while (sptr->n_name != NULL)
    {
	if (sptr->n_value == 0 && strcmp (sptr->n_name, SYM_DUMMY) != 0)
	{
	    if (check)
		wprintw (wscr, "kernel: no symbol named `%s'\n", sptr->n_name);
	    failed++;
	}
	sptr++;
    }

    if (check && failed != 0)
	LeaveText ("cannot get all kmem symbols - goodbye.", 1);

    return (failed != 0);
}


static void PageHandler (int sig, siginfo_t *sinfo, ucontext_t *uctx)
{
    caddr_t     page;
    caddr_t     pa;

    if (kmem_fd < 0 || sinfo == (siginfo_t *) 0 || !SI_FROMKERNEL (sinfo))
	LeaveText ("Unknown signal passed to PageHandler()", 1);

    if (UTKT (sinfo->si_addr, caddr_t) < (caddr_t) KERNEL_VS_MIN
	|| UTKT (sinfo->si_addr, caddr_t) > (caddr_t) KERNEL_VS_MAX
    )
	LeaveText ("SIGSEGV on no-kernel address", 1);

#ifdef DEBUG
    fprintf (stderr,
	"failed: 0x%x (0x%x)\n", sinfo->si_addr, UTK(sinfo->si_addr)
    );
#endif

    page = (caddr_t) (UTKT(sinfo->si_addr, unsigned long)
		& ~((unsigned long) NBPP - 1));

#ifdef DEBUG
    fprintf (stderr, "map: 0x%x (0x%x)\n", KTU (page), page);
#endif

    pa = mmap (KTUT (page, caddr_t),
	NBPP, PROT_READ, MAP_FIXED | MAP_SHARED, kmem_fd, (off_t) page
    );

    if (pa == (caddr_t) -1)
	LeaveText ("mmap() failed", 255);
}


void SetupPaging (void)
{
    static struct sigaction     new_act;
    static struct nlist kmemRange[] = {
#if defined(S_SINIX_MIPS)
	{ "start"  },
#else
	{ "_start" },
#endif
	{ "_end"   },
	{ 0        }
    };

    if (nlist (VMUNIX_FILE, kmemRange) != -1)
    {
	minkvadr = (caddr_t) kmemRange[0].n_value;
	maxkvadr = (caddr_t) kmemRange[1].n_value;

	if (kmemRange[0].n_value == 0)
	    LeaveText ("Cannot get address for _start (nlist)", 255);
	if (kmemRange[1].n_value == 0)
	    LeaveText ("Cannot get address for _end (nlist)", 255);
    }
    else
    {
#if defined(S_MOTOROLA_88K)
	minkvadr = (caddr_t) KERNBASE;
	maxkvadr = (caddr_t) 0x00500000;
#elif defined(S_SINIX_MIPS)
	minkvadr = (caddr_t) KTXTSTRT;
	maxkvadr = (caddr_t) ((unsigned) 0xE0000000L);
#else
	minkvadr = (caddr_t) KVSBASE;
	maxkvadr = (caddr_t) MAXKVADR;
#endif
    }

    /*
    **  for now, we cannot use _end as max kernel VM address, because we
    **  needn't only know the max data address, we have to know the max
    **  dynamic memory address. So we reset the max address here.
    */
#if defined(S_MOTOROLA_88K)
    maxkvadr = (caddr_t) 0x01000000; /* FIXME: just take a reasonable value */
#elif defined(S_SINIX_MIPS)
    maxkvadr = (caddr_t) ((unsigned) 0xE0000000L); /* FIXME: just take a reasonable value */
#else
    maxkvadr = (caddr_t) MAXKVADR;
#endif

#ifdef S_UNIVEL
    /*
    **  On UnixWare just set the minimum and maximum kernel virtual addresses
    **  to the values given in the header file. This should be O.K.
    **  So, for now, overwrite the computed values, otherwise it will become
    **  very complicated with all these nesting #ifdef's.
    */
    minkvadr = (caddr_t) MINKVADR;
    maxkvadr = (caddr_t) MAXKVADR;
#endif

    if (kmem_fd < 0)
	if ((kmem_fd = open (KMEM_FILE, O_RDONLY, 0)) < 0)
	    LeaveText ("Cannot open kmem", 255);

    new_act.sa_handler = PageHandler;
    new_act.sa_flags   = SA_RESTART | SA_SIGINFO;
    sigemptyset (&new_act.sa_mask);

    if (sigaction (SIGSEGV, &new_act, 0) == -1)
	LeaveText ("sigaction:", 255);

    /*
    **  This part is really tricky: We have to know at what address new
    **  mapped pages will start. I see no way to get this information
    **  other than just mapping a dummy page and then remember the start
    **  address. This dummy page will be unmapped immediatly after this check.
    **  This way, we have to assume, that the first page mapped will start
    **  exactly at this address and that *every* other page following
    **  will be mapped contigous to the end of the previous page.
    **  Otherwise the whole mapping algorithm will not function!!
    */
    if (pagingBase == (caddr_t) 0)
    {
	pagingBase = mmap (
	    0, NBPP, PROT_READ, MAP_SHARED, kmem_fd, (off_t) KERNEL_VS_MIN
	);

	if (pagingBase == (caddr_t) -1)
	    LeaveText ("SetupPaging (mmap)", 255);

	(void) munmap (pagingBase, NBPP);
    }
}
