/*
**
**                                P R O C . C
**
**
**      Module:     process display routines
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      09-OCT-1992 AV  Version X1.0, first definition
**      16-OCT-1992 AV  fixed wrong proc size calculation, display it
**                      in pages
**      18-OCT-1992 AV  implemented page scroll
**      18-OCT-1992 AV  implemented short command display
**      19-OCT-1992 AV  fixed some typos (pri, nice, size)
**      21-OCT-1992 AV  start port for ARCH_MOT_88k
**      23-OCT-1992 AV  Version 1.0
**      25-OCT-1992 AV  show total number of pages in bottom line
**      11-NOV-1993 AV  added MIPS support and LOGGING support
**
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>

#include <curses.h>
#include <panel.h>
#include <utmp.h>

#include <sys/sysmacros.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/proc.h>
#include <sys/ts.h>
#include <sys/param.h>
#include <sys/class.h>
#include <sys/vm.h>
#include <sys/user.h>
#include <sys/var.h>
#include <sys/cred.h>
#include <sys/session.h>
#include <vm/as.h>
#include <vm/anon.h>

#if defined(S_MOTOROLA_88K)
#   include <sys/sysm88k.h>
#elif defined(S_SINIX_MIPS)
#   include <sys/sysmips.h>
#else
#   include <sys/sysi86.h>
#endif

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"
#include "uidutil.h"

#if defined(S_SINIX_MIPS)
#   define ADDR_class           kmemSymbols[0].n_value
#   define ADDR_pidhash         kmemSymbols[1].n_value
#   define ADDR_v               kmemSymbols[2].n_value
#   define ADDR_procdir         kmemSymbols[3].n_value

    static struct nlist kmemSymbols[] =
    {
	{ "class"               },
	{ "pidhash"             },
	{ "v"                   },
	{ "procdir"             },
	{ 0                     },
    };
#else
#   define ADDR_class           kmemSymbols[0].n_value
#   define ADDR_practive        kmemSymbols[1].n_value
#   define ADDR_v               kmemSymbols[2].n_value

    static struct nlist kmemSymbols[] =
    {
	{ "class"               },
	{ "practive"            },
	{ "v"                   },
	{ 0                     },
    };
#endif

#if PSCOMSIZ > PSARGSZ
#   define CMDLEN   PSCOMSIZ + 1
#else
#   define CMDLEN   PSARGSZ + 1
#endif

#define NICE_RT    9000
#define NICE_SYS   9001

typedef struct prin
{
    char        stat;
    char        cpu;
    u_int       flag;
    int         pri;
    clock_t     utime;
    clock_t     stime;
    pid_t       sid;
    pid_t       pid;
    uid_t       uid;
    uid_t       suid;
    int         nice;
    char        cmd[CMDLEN];
    u_long      size;
    u_long      rssize;
    int         gotUBlock;
    int         useCp;

} prin_t;

/*
**  current page to display
*/
static int          procPage = 0;
/*
**  show short/long commands
*/
static int          shortCmd = 0;

static struct prin  *curPrinTable  = (struct prin *) 0;
static int          nCurPrin       = 0;

static struct prin  *oldPrinTable  = (struct prin *) 0;
static int          nOldPrin       = 0;

#define MAX_UTMP    64
static int          nutmps = 0;
static struct utmp  utmps[MAX_UTMP];
static int          utmpPass;
static time_t       utmpLastTime = (time_t) 0;

static void ReadUtmp ()
{
    struct utmp     *tutmp = utmps;
    struct utmp     *ptr;
    struct stat     statbuf;

    (void) stat (UTMP_FILE, &statbuf);

    if (statbuf.st_mtime == utmpLastTime)
	return;
    else
	utmpLastTime = statbuf.st_mtime;

    nutmps = 0;

    while ((ptr = getutent ()) != (struct utmp *) 0)
    {
	*tutmp++ = *ptr;
	tutmp->ut_id[sizeof(tutmp->ut_id)] = 0;

	if (++nutmps == MAX_UTMP)
	    LeaveText ("too many utmp entries for me to handle", 1);
    }

    endutent ();
}

static struct utmp *FindUtmpForPgrp (pid_t pgrp)
{
    struct utmp     *tutmp = utmps;
    int             count = nutmps;

    while (count--)
    {
	if (tutmp->ut_pid == pgrp)
	    return (tutmp);
	tutmp++;
    }
    return ((struct utmp *) 0);
}


static char *PgrpToTtyname (pid_t pgrp)
{
    int             itmp;
    struct utmp     *tutmp = (struct utmp *) 0;

    if (!(tutmp = FindUtmpForPgrp (pgrp)) && utmpPass > 0)
    {
	ReadUtmp ();
	tutmp = FindUtmpForPgrp (pgrp);
	utmpPass--;
    }

    if (!tutmp)
	return ("??");

    itmp = strlen (tutmp->ut_id);
    return (&tutmp->ut_id[(itmp >= 2) ? (itmp - 2) : 0]);
}

/*
 *  proc_compare - comparison function for "qsort"
 *      Compares the resource consumption of two processes using five
 *      distinct keys.  The keys (in descending order of importance) are:
 *      percent cpu, cpu ticks, state, resident set size, total virtual
 *      memory usage.  The process states are ordered as follows (from least
 *      to most important):  WAIT, zombie, sleep, stop, start, run.  The
 *      array declaration below maps a process state index into a number
 *      that reflects this ordering.
 */

static unsigned char sorted_state[] =
{
    0,  /* not used             */
    3,  /* sleep                */
    6,  /* run                  */
    2,  /* zombie               */
    4,  /* stop                 */
    5,  /* start                */
    7,  /* run on a processor   */
    1   /* being swapped (WAIT) */
};

static int PrinCompare (const void *pp1, const void *pp2)
{
    struct prin    *p1 = (struct prin *) pp1;
    struct prin    *p2 = (struct prin *) pp2;
    long           result;

    if (p1->pid == 1)
	return (1);

    if (p2->pid == 1)
	return (-1);

    if (p1->flag & SSYS && !(p2->flag & SSYS))
	return (1);

    if (p2->flag & SSYS && !(p1->flag & SSYS))
	return (-1);

    if ((result = (long) (p2->cpu - p1->cpu)) == 0)
    {
	if ((result = p2->utime - p1->utime) == 0)
	{
	    if ((result = (long) (sorted_state[p2->stat] - sorted_state[p1->stat]))  == 0)
	    {
		if ((result = p2->pri - p1->pri) == 0)
		{
		    result = p2->size - p1->size;
		}
	    }
	}
    }

    return (result == 0L ? 0 : result < 0L ? -1 : 1);
}


static char *GetCpuTimeString (time_t ticks)
{
    static char     timestr[10];
    time_t          mm, ss;

    if (ticks < 0)
	return ("------");

    ticks /= hertz;
    mm = ticks / 60L;
    ticks -= mm * 60L;
    ss = ticks;

    if (mm > 9999)
	(void) strcpy (timestr, ">9999m");
    else if (mm > 999)
	(void) sprintf (timestr, "%5ldm", mm);
    else
	(void) sprintf (timestr, "%3lu:%02lu", mm, ss);

    return (timestr);
}

static int GetNice (id_t class, void *clproc)
{
    class_t     *classTable = KTUT (ADDR_class, class_t *);

    if (!strcmp ("TS", KTUT (classTable[class].cl_name, char *)))
    {
	tsproc_t    *ts = KTUT (clproc, tsproc_t *);

	return (ts->ts_nice - NZERO);
    }
    else if (!strcmp ("RT", KTUT (classTable[class].cl_name, char *)))
	return (NICE_RT);
    else if (!strcmp ("SYS", KTUT (classTable[class].cl_name, char *)))
	return (NICE_SYS);

    return (0);
}

static void BuildPrin ()
{
    struct proc     *pproc;
    struct prin     *pprin;
    struct sess     *sessp;
    struct as       *pas;
    struct user     puser;
    int             idx;

    if (curPrinTable == (struct prin *) 0)
    {
	struct var  *v = KTUT (ADDR_v, struct var *);

	curPrinTable = (struct prin *) calloc (v->v_proc, sizeof (struct prin));
	if (curPrinTable == (struct prin *) 0)
	    LeaveText ("Cannot perform malloc for curPrinTable", 1);
	nCurPrin = 0;

	oldPrinTable = (struct prin *) calloc (v->v_proc, sizeof (struct prin));
	if (oldPrinTable == (struct prin *) 0)
	    LeaveText ("Cannot perform malloc for oldPrinTable", 1);
	nOldPrin = 0;
    }
    else
    {
	(void) memcpy (
	    (char *) oldPrinTable, (char *) curPrinTable, nCurPrin  * sizeof (struct prin)
	);
	nOldPrin = nCurPrin;
    }

#if defined(S_SINIX_MIPS)
    {
	union procent
	{
	    proc_t          *pe_proc;
	    union procent   *pe_next;
	};

	struct pid          *pidp;
	union procent       *procdir;
	struct pid          **pidhash;
	int                 hash_index;

	idx   = 0;
	pprin = curPrinTable;

	pidhash = KTUT (ADDR_pidhash, struct pid **);
	procdir = KTUT (ADDR_procdir, union procent *);

	for (hash_index = 0; hash_index < v->v_phash; hash_index++)
	{
	    for (pidp = KTUT (pidhash[hash_index], struct pid *);
		pidp;
		pidp = KTUT (pidp->pid_link, struct pid *)
	    )
	    {
		if (pidp == (struct pid *) NULL)
		{
		    hash_index++;           /* goto next hash */
		    continue;
		}
		if (pidp->pid_prinactive)
		{
		    continue;       /* goto nextlink */
		}

		pproc = KTUT (
		    procdir[pidp->pid_prslot].pe_proc, struct proc *
		);

		if (pproc->p_stat == 0)
		    continue;

		pprin->pri   = pproc->p_pri;
		pprin->stat  = pproc->p_stat;
		pprin->cpu   = pproc->p_cpu;
		pprin->flag  = pproc->p_flag;
		pprin->utime = pproc->p_utime;
		pprin->stime = pproc->p_stime;
		pprin->pid   = KTUT (pproc->p_pidp, struct pid *)->pid_id;
		pprin->uid   = KTUT (pproc->p_cred, struct cred *)->cr_uid;
		pprin->suid  = KTUT (pproc->p_cred, struct cred *)->cr_suid;
		sessp        = KTUT (pproc->p_sessp, struct sess *);
		pprin->sid   = KTUT (sessp->s_sidp, struct pid *)->pid_id;
		pprin->nice  = GetNice (pproc->p_cid, pproc->p_clproc);

		pas = KTUT (pproc->p_as, struct as *);
		if (pas != (struct as *) 0)
		{
		    pprin->size   = bytestopage (pas->a_size);
		    pprin->rssize = pas->a_rss;
		}
		else
		{
		    pprin->size = pprin->rssize = ~0;
		}

		if (shortCmd)
		    (void) strncpy (
			pprin->cmd, KTUT (pproc->p_comm, char *), PSCOMSIZ
		    );
		else
		{
		    char    *tmp = KTUT (pproc->p_psargs, char *);

		    (void) strncpy (
			pprin->cmd, KTUT (pproc->p_psargs, char *), PSARGSZ
		    );
		}

		pproc = KTUT (pproc->p_next, struct proc *);
		pprin++;
		idx++;
	    }
	}
    }
#else
    pproc = KTUT (*KTUT (ADDR_practive, struct proc **), struct proc *);

    idx   = 0;
    pprin = curPrinTable;

    while (pproc != (struct proc *) 0)
    {
	pprin->pri   = pproc->p_pri;
	pprin->stat  = pproc->p_stat;
	pprin->cpu   = pproc->p_cpu;
	pprin->flag  = pproc->p_flag;
	pprin->utime = pproc->p_utime;
	pprin->stime = pproc->p_stime;
	pprin->pid   = KTUT (pproc->p_pidp, struct pid *)->pid_id;
	pprin->uid   = KTUT (pproc->p_cred, struct cred *)->cr_uid;
	pprin->suid  = KTUT (pproc->p_cred, struct cred *)->cr_suid;
	sessp        = KTUT (pproc->p_sessp, struct sess *);
	pprin->sid   = KTUT (sessp->s_sidp, struct pid *)->pid_id;
	pprin->nice  = GetNice (pproc->p_cid, pproc->p_clproc);

	pas = KTUT (pproc->p_as, struct as *);
	if (pas != (struct as *) 0)
	{
	    pprin->size   = bytestopage (pas->a_size);
	    pprin->rssize = pas->a_rss;
	}
	else
	    pprin->size = pprin->rssize = ~0;

/*
**  don't undefine this, it still remains for testing
*/
#define USE_RDUBLK

#if defined(USE_RDUBLK)
#   if defined(S_MOTOROLA_88K)
	pprin->gotUBlock = !!sysm88k (
	    RDUBLK, pprin->pid, (char *) &puser, sizeof (struct user)
	);
#   else
	pprin->gotUBlock = !!sysi86 (
	    RDUBLK, pprin->pid, (char *) &puser, sizeof (struct user)
	);
#   endif
#else
#   ifdef RAIMUND
	if (pproc->p_flag & SULOAD)
	{
	    (void) memcpy (
		(void *) &puser,
		(void *) KTUT (PTOU (pproc), struct user *),
		sizeof (struct user)
	    );
	    pprin->gotUBlock = 1;
	}
	else
	    pprin->gotUBlock = 0;
#   else
	if (pproc->p_flag & SULOAD
	    && pproc->p_stat != SZOMB
	    && pproc->p_stat != SXBRK
	    && pproc->p_stat != SIDL
	)
	{
	    struct user     *u;

	    u = KTUT (PTOU (pproc), struct user *);
	    (void) memcpy ((void *) &puser, (void *) u, sizeof (struct user));
	    pprin->gotUBlock = 1;
	}
	else
	    pprin->gotUBlock = 0;
#   endif
#endif

	if (pprin->gotUBlock)
	{
	    if (shortCmd)
		(void) strncpy (pprin->cmd, puser.u_comm, PSCOMSIZ);
	    else
		(void) strncpy (pprin->cmd, puser.u_psargs, PSARGSZ);
	}
	else
	    *pprin->cmd = '\0';

	pproc = KTUT (pproc->p_next, struct proc *);
	pprin++;
	idx++;
    }
#endif

    nCurPrin = idx;

    (void) qsort (
	(char *) curPrinTable, (unsigned) nCurPrin, sizeof (struct prin),
	PrinCompare
    );
}

/*
**  00000000001111111111222222222233333333334444444444555555555566666666667777777777
**  01234567890123456789012345678901234567890123456789012345678901234567890123456789
**  S     USER    PID CPU PRI  NI   UCPU   SCPU SIZE RSIZ TTY COMMAND (short)
**  #!########X ##### ### ### ### ###### ###### #### #### ### ######################
*/

#define PROC_Y          1
#define PROC_X          0
#define UID_X           2
#define PID_X           12
#define CPU_X           18
#define PRI_X           22
#define NICE_X          26
#define UTIME_X         30
#define STIME_X         37
#define SIZE_X          44
#define RSSIZE_X        49
#define TTY_X           54
#define CMD_X           58

static void ShowPrin (WINDOW *win, int pidx, int lidx, int initial)
{
    struct prin     *cprin      = &curPrinTable[pidx];
    struct prin     *oprin      = &oldPrinTable[pidx];
    int             positioned  = 0;
    static char     *p_stat_str = " sRzdipx";
    char            buf[64];

    if (cprin->useCp != oprin->useCp)
	initial = 1;

    if (cprin->stat == SRUN && !(cprin->flag & SLOAD))
    {
	use_cp (win, cpHIGH);
	cprin->useCp = cpHIGH;
    }
    else if (cprin->stat == SRUN)
    {
	use_cp (win, cpMED);
	cprin->useCp = cpMED;
    }
    else
    {
	use_cp (win, cpINFO);
	cprin->useCp = cpINFO;
    }

    if (initial || cprin->stat != oprin->stat || cprin->flag != oprin->flag)
    {
	if (!positioned)
	    wmove (win, PROC_Y + lidx, PROC_X);
	waddch (win, (chtype) p_stat_str[cprin->stat]);
	waddch (win, cprin->flag & SLOAD ? (chtype) ' ' : (chtype) 'S');
	positioned = 1;
    }
    else
	positioned = 0;

    if (initial || cprin->uid != oprin->uid || cprin->suid != oprin->suid)
    {
	if (!positioned)
	    wmove (win, PROC_Y + lidx, PROC_X + UID_X);
	(void) sprintf (buf, "%8.8s", UidToName (cprin->uid));
	waddstr (win, buf);
	waddch (win, cprin->uid != cprin->suid ? '#' : ' ');
	waddch (win, (chtype) ' ');
	positioned = 1;
    }
    else
	positioned = 0;

    if (initial || cprin->pid != oprin->pid)
    {
	if (!positioned)
	    wmove (win, PROC_Y + lidx, PROC_X + PID_X);
	if (cprin->pid > 99999)
	    (void) strcpy (buf, "?????  ");
	else
	    (void) sprintf (buf, "%5d ", cprin->pid);
	waddstr (win, buf);
	positioned = 1;
    }
    else
	positioned = 0;

    if (initial || cprin->cpu != oprin->cpu)
    {
	if (!positioned)
	    wmove (win, PROC_Y + lidx, PROC_X + CPU_X);
	(void) sprintf (buf, "%3u ", cprin->cpu);
	waddstr (win, buf);
	positioned = 1;
    }
    else
	positioned = 0;

    if (initial || cprin->pri != oprin->pri)
    {
	if (!positioned)
	    wmove (win, PROC_Y + lidx, PROC_X + PRI_X);
	if (cprin->pri < -99 || cprin->pri > 999)
	    (void) strcpy (buf, "???  ");
	else
	    (void) sprintf (buf, "%3d ", cprin->pri);
	waddstr (win, buf);
	positioned = 1;
    }
    else
	positioned = 0;

    if (initial || cprin->nice != oprin->nice)
    {
	if (!positioned)
	    wmove (win, PROC_Y + lidx, PROC_X + NICE_X);
	if (cprin->nice == NICE_RT)
	    (void) strcpy (buf, " RT ");
	else if (cprin->nice == NICE_SYS)
	    (void) strcpy (buf, "SYS ");
	else if (cprin->nice < -99 || cprin->nice > 999)
	    (void) strcpy (buf, "??? ");
	else
	    (void) sprintf (buf, "%3d ", cprin->nice);
	waddstr (win, buf);
	positioned = 1;
    }
    else
	positioned = 0;

    if (initial || cprin->utime != oprin->utime)
    {
	if (!positioned)
	    wmove (win, PROC_Y + lidx, PROC_X + UTIME_X);
	waddstr (win, GetCpuTimeString (cprin->utime));
	waddch (win, ' ');
	positioned = 1;
    }
    else
	positioned = 0;

    if (initial || cprin->stime != oprin->stime)
    {
	if (!positioned)
	    wmove (win, PROC_Y + lidx, PROC_X + STIME_X);
	waddstr (win, GetCpuTimeString (cprin->stime));
	waddch (win, ' ');
	positioned = 1;
    }
    else
	positioned = 0;

    if (initial || cprin->size != oprin->size)
    {
	if (!positioned)
	    wmove (win, PROC_Y + lidx, PROC_X + SIZE_X);

	if (cprin->size > 9999)
	    (void) strcpy (buf, "???? ");
	else
	    (void) sprintf (buf, "%4lu ", cprin->size);
	waddstr (win, buf);
	positioned = 1;
    }
    else
	positioned = 0;

    if (initial || cprin->rssize != oprin->rssize)
    {
	if (!positioned)
	    wmove (win, PROC_Y + lidx, PROC_X + RSSIZE_X);

	if (cprin->rssize > 9999)
	    (void) strcpy (buf, "???? ");
	else
	    (void) sprintf (buf, "%4lu ", cprin->rssize);
	waddstr (win, buf);
	positioned = 1;
    }
    else
	positioned = 0;

    if (initial || cprin->sid != oprin->sid)
    {
	if (!positioned)
	    wmove (win, PROC_Y + lidx, PROC_X + TTY_X);
	(void) sprintf (buf, "%3.3s ", PgrpToTtyname (cprin->sid));
	waddstr (win, buf);
	positioned = 1;
    }
    else
	positioned = 0;

    if (initial || !strcmp (cprin->cmd, oprin->cmd) || cprin->gotUBlock != oprin->gotUBlock)
    {
	if (!positioned)
	    wmove (win, PROC_Y + lidx, PROC_X + CMD_X);

	if (cprin->gotUBlock)
	{
	    char   *cptr;
	    int    y, x, maxx;

	    maxx = getmaxx (win);
	    getyx (win, y, x);

	    for (cptr = cprin->cmd; *cptr && x < maxx; cptr++, x++)
	    {
		*cptr &= 0x7F;
		if (*cptr < ' ')
		    *cptr = ' ';
	    }

	    *cptr = '\0';
	    waddstr (win, cprin->cmd);
	}
	else
	{
	    switch (cprin->stat)
	    {
		case SZOMB:
		    waddstr (win, "<zombie>");
		    break;
		case SXBRK:
		    waddstr (win, "<xbreak>");
		    break;
		case SIDL:
		    waddstr (win, "<in creation>");
		    break;
		default:
		    waddstr (win, "<swapping>");
	    }
	}
    }

    wclrtoeol (win);
}

#define LASTPAGE    99999

void ProcCommand (int cmd)
{
    if (cmd >= '0' && cmd <= '9')
	procPage = cmd - '0';
    else if (cmd == ',' || cmd == KEY_UP || cmd == KEY_PREVIOUS)
	procPage -= (procPage > 0) ? 1 : 0;
    else if (cmd == '.' || cmd == KEY_DOWN || cmd == KEY_NEXT)
	procPage += 1;
    else if (cmd == '<' || cmd == KEY_LEFT || cmd == KEY_BEG)
	procPage = 0;
    else if (cmd == '>' || cmd == KEY_RIGHT || cmd == KEY_END)
	procPage = LASTPAGE;
    else if (cmd == '@')
	shortCmd = !shortCmd;
    else
    {
	procPage = 0;
	beep ();
    }
}


void DspProc (WINDOW *win, int initial)
{
    int     idx;
    int     start;
    int     stop;
    int     height;
    int     numOmitted;
    int     x;
    int     y;

    touchwin (win);
    initial = 1;

    if (initial)
    {
	use_cp (win, cpBANNER);

	wmove (win, 0, 0);
	waddstr (win,
	    "S     USER    PID CPU PRI  NI   UCPU   SCPU SIZE RSIZ TTY COMMAND"
	);

	if (shortCmd)
	    waddstr (win, " (short)");
	else
	    waddstr (win, " (long) ");

	getyx (win, y, x);
	while (x < getmaxx (win))
	    waddch (win, (chtype) ' '), x++;
    }

    BuildPrin ();

    height = getmaxy (win) - PROC_Y - 1;

    if (procPage < 0)
	procPage = 0;
    else if (procPage == LASTPAGE)
	procPage = (nCurPrin - 1) / height;

    start = procPage * height;
    stop  = start + height - 1;

    if (start >= nCurPrin)
    {
	procPage = (nCurPrin - 1) / height;
	start    = procPage * height;
	stop     = start + height - 1;
    }

    if (stop >= nCurPrin)
	stop = nCurPrin - 1;

    utmpPass = 1;
    for (idx = start; idx <= stop; idx++)
	ShowPrin (win, idx, idx - start, initial);

    wclrtobot (win);

    if ((numOmitted = nCurPrin - (stop - start + 1)) < 0)
	numOmitted = 0;

    use_cp (win, cpLIT);
    mvwprintw (win, getmaxy (win) - 1, 0,
	"Page: %d of %d Procs: %d - %d Total: %d Omitted: %d",
	procPage, (nCurPrin - 1) / height,
	start + 1, stop + 1, nCurPrin, numOmitted
    );
    getyx (win, y, x);
    while (x < getmaxx (win))
	waddch (win, (chtype) ' '), x++;

    wmove (win, getmaxy (win) - 1, getmaxx (win) - 1);
}

static char *PstatText (int pstat)
{
    static char errant[10];

    switch (pstat)
    {
	case SSLEEP:   return ("sleep     ");
	case SRUN:     return ("run       ");
	case SZOMB:    return ("zombie    ");
	case SSTOP:    return ("stop      ");
	case SIDL:     return ("idle      ");
	case SONPROC:  return ("onproc    ");
	case SXBRK:    return ("xbrk      ");
    }

    (void) sprintf (errant, "%06.6u?" , (unsigned char) pstat);

    return (errant);
}


void DspProcSums (WINDOW *win, int y, int x)
{
    static int      procs_per_pstat[SXBRK + 1];
    struct proc     *pproc;
    int             idx;
    int             procs_alive;
    int             procs_in_core;

    (void) memset (procs_per_pstat, 0, sizeof (procs_per_pstat));
    procs_in_core = 0;
    procs_alive   = 0;

#if !defined(S_SINIX_MIPS)
    pproc = KTUT (*KTUT (ADDR_practive, struct proc **), struct proc *);

    idx = 0;
    while (pproc != (struct proc *) 0)
    {
	procs_per_pstat[pproc->p_stat]++;

	if (pproc->p_stat)
	    procs_alive++;

	if (pproc->p_flag & SLOAD)
	    procs_in_core++;

	pproc = KTUT (pproc->p_next, struct proc *);
    }
#endif

    use_cp (win, cpBANNER);
    wmove (win, y++, x);
    waddstr (win, "----- Proc ----");

    for (idx = SSLEEP; idx <= SXBRK; idx++)
    {
	wmove (win, y++, x);
	DispInfoInt (win, PstatText (idx), "%5d", procs_per_pstat[idx]);
    }

    use_cp (win, cpLIT);
    mvwaddstr (win, y++, x, "---------------");

    wmove (win, y++, x);
    DispInfoInt (win, "total     ", "%5d", procs_alive);

    wmove(win,y++,x);
    DispInfoInt (win, "in mem    ", "%5d", procs_in_core);
    wmove (win, getmaxy (win) - 1, getmaxx (win) - 1);
}

void ProcInit ()
{
    (void) GetSymbols (kmemSymbols, 1);
    procPage = 0;
    shortCmd = 0;
    InitUidToNameHash ();
}
