/*
**
**                                S L I P . C
**
**
**      Module:     display of SLIP statistics
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      24-OCT-1992 AV  Version X1.0, first definition
**      15-NOV-1993 AV  fixed positioning bug in DspSlipLiterals
**
*/

#include "config.h"

#if defined(MODULE_SLIP)

#include <curses.h>
#include <panel.h>

#include <sys/types.h>
#include <sys/stream.h>
#include <sys/socket.h>
#include <net/if.h>

#include <netinet/in_systm.h>
/*
**  damn - name space pollution
*/
#undef MIN
#undef MAX

#include <netinet/in.h>
#include <netinet/ip.h>
#include <sys/dlpi.h>
#include <sys/slp.h>

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"

#define ADDR_slip_num               kmemSymbols[0].n_value
#define ADDR_slip_data              kmemSymbols[1].n_value

static struct nlist kmemSymbols[] =
{
    { "slip_num"            },
    { "slip_data"           },
    { 0                     },
};

static slip_t   *curSlip = (slip_t *) 0;
static slip_t   *oldSlip = (slip_t *) 0;
static u_int    slip_num = 0;

static int      noSyms;

#define SL_DELTA(x) (infoMode == modeAbsolute ? slp->x : slp->x - olp->x)
#define SC_DELTA(x) (infoMode == modeAbsolute ? slp->sc_comp.x : slp->sc_comp.x - olp->sc_comp.x)
#define ST_DELTA(x) (infoMode == modeAbsolute ? slp->stats.x : slp->stats.x - olp->stats.x)


/*
**  00000000001111111111222222222233333333334444444444555555555566666666667777777777
**  01234567890123456789012345678901234567890123456789012345678901234567890123456789
**  ---- SLIP statistics --- -------------------------------------------------------
**  Name       Mtu Network     Address      Flags  Ipkts  Ierrs  Opkts  Oerrs Collis
**  ######## ##### ########### ############ ##### ###### ###### ###### ###### ######
**  ######## ##### ########### ############ ##### ###### ###### ###### ###### ######
**  ######## ##### ########### ############ ##### ###### ###### ###### ###### ######
**  ######## ##### ########### ############ ##### ###### ###### ###### ###### ######
**  ######## ##### ########### ############ ##### ###### ###### ###### ###### ######
**  ######## ##### ########### ############ ##### ###### ###### ###### ###### ######
**   hisnd  hircv uwblck   pkts   cmpr  srch  miss   ucmpin    cmpin  errin tossin
**  ###### ###### ###### ###### ###### ##### ##### ######## ######## ###### ######
**  ###### ###### ###### ###### ###### ##### ##### ######## ######## ###### ######
**  ###### ###### ###### ###### ###### ##### ##### ######## ######## ###### ######
**  ###### ###### ###### ###### ###### ##### ##### ######## ######## ###### ######
**  ###### ###### ###### ###### ###### ##### ##### ######## ######## ###### ######
**  ###### ###### ###### ###### ###### ##### ##### ######## ######## ###### ######
**
*/

/*
** packets         slp->sc_comp.sls_packets           outbound packets
** compr           slp->sc_comp.sls_compressed        outbound compressed packets
** search          slp->sc_comp.sls_searches          searches for connection state
** miss            slp->sc_comp.sls_misses            times couldn't find conn. state
** uncompin        slp->sc_comp.sls_uncompressedin    inbound uncompressed packets
** compin          slp->sc_comp.sls_compressedin      inbound compressed packets
** errorin         slp->sc_comp.sls_errorin           inbound unknown type packets
** tossedin        slp->sc_comp.sls_tossed            inbound packets tossed because of error
**
** sc_flags        slp->sc_flags
** hisend          slp->xmit_hiband                   # of hiband xmissions
** hirecv          slp->recv_hiband                   and receptions
** uwblocked       slp->uw_blocked                    # of uwsrv blockages
*/

static char *ShowFlags (WINDOW *win, u_int flag)
{
    if (flag & IFF_UP) use_cp (win, cpREVERSE); else use_cp (win, cpINFO);
    waddch (win, 'U');

    if (flag & IFF_POINTOPOINT) use_cp (win, cpREVERSE); else use_cp (win, cpINFO);
    waddch (win, 'P');

    if (flag & IFF_RUNNING) use_cp (win, cpREVERSE); else use_cp (win, cpINFO);
    waddch (win, 'R');

    if (flag & IFF_NOARP) use_cp (win, cpREVERSE); else use_cp (win, cpINFO);
    waddch (win, 'A');

    if (flag & IFF_NOTRAILERS) use_cp (win, cpREVERSE); else use_cp (win, cpINFO);
    waddch (win, 'C');

    use_cp (win, cpINFO);
}


void SlipModeChange ()
{
    if (noSyms || slip_num == 0)
	return;

    switch (infoMode)
    {
	case modeDelta:
	case modeRelative:
	    (void) memcpy (oldSlip, curSlip, slip_num * sizeof (slip_t));
	    break;
	case modeAbsolute:
	    break;
    }
}


void DspSlipLiterals (WINDOW *win, int ty, int tx)
{
    static char *header1 = "Name       Mtu Network     Address      Flags  Ipkts  Ierrs  Opkts  Oerrs Collis";
    static char *header2 = " hisnd  hircv uwblck   pkts   cmpr  srch  miss   ucmpin    cmpin  errin tossin  ";
    int         x;
    int         y;

    wmove (win, ty, tx);
    use_cp (win, cpBANNER);
    waddstr (win, "---- SLIP statistics --- ");
    getyx (win, y, x);
    while (x++ < getmaxx (win))
	waddch (win, (chtype) '-');

    if (noSyms)
    {
	char    *message = "SLIP related kernel variables not found!";
	int     x;

	use_cp (win, cpBANWARN);
	mvwaddstr (win,
	    (getmaxy (win) - 1) / 2,
	    (getmaxx (win) - strlen (message)) / 2,
	    message
	);

	wclrtoeol (win);
	wclrtobot (win);
	return;
    }

    wmove (win, ty + 1, tx);
    use_cp (win, cpLIT);
    waddstr (win, header1);

    wmove (win, ty + 8, tx);
    use_cp (win, cpLIT);
    waddstr (win, header2);
}


static int ShowSlipEntry (WINDOW *win, int ty, int tx, int line, int idx)
{
    char    name[32];
    slip_t  *slp = curSlip + idx;
    slip_t  *olp = oldSlip + idx;
    int     y;

    if (*slp->uname == '\0' || !slp->buf)
    {
	wmove (win, ty + 2 + line, tx); wclrtoeol (win);
	wmove (win, ty + 9 + line, tx); wclrtoeol (win);
	return (0);
    }

    use_cp (win, cpINFO);

    y = ty + 2 + line;
    (void) sprintf (name, "%s%d", slp->uname, slp->stats.ifs_unit);
    mvwprintw (win, y, tx +  0, "%-8.8s",   name);
    mvwprintw (win, y, tx +  9, "%5d",      (int) slp->stats.ifs_mtu);
    mvwprintw (win, y, tx + 15, "%-11.11s", "");
    mvwprintw (win, y, tx + 27, "%-12.12s", "");

    wmove (win, y, tx + 40);
    ShowFlags (win, (u_int) slp->flags);

    mvwprintw (win, y, tx + 46, "%6d", (int) ST_DELTA (ifs_ipackets));
    mvwprintw (win, y, tx + 53, "%6d", (int) ST_DELTA (ifs_ierrors));
    mvwprintw (win, y, tx + 60, "%6d", (int) ST_DELTA (ifs_opackets));
    mvwprintw (win, y, tx + 67, "%6d", (int) ST_DELTA (ifs_oerrors));
    mvwprintw (win, y, tx + 74, "%6d", (int) ST_DELTA (ifs_collisions));

    y = ty + 9 + line;
    mvwprintw (win, y, tx +  0, "%6d", (int) SL_DELTA (xmit_hiband));
    mvwprintw (win, y, tx +  7, "%6d", (int) SL_DELTA (recv_hiband));
    mvwprintw (win, y, tx + 14, "%6d", (int) SL_DELTA (uw_blocked));
    mvwprintw (win, y, tx + 21, "%6d", (int) SC_DELTA (sls_packets));
    mvwprintw (win, y, tx + 28, "%6d", (int) SC_DELTA (sls_compressed));
    mvwprintw (win, y, tx + 35, "%5d", (int) SC_DELTA (sls_searches));
    mvwprintw (win, y, tx + 41, "%5d", (int) SC_DELTA (sls_misses));
    mvwprintw (win, y, tx + 47, "%8d", (int) SC_DELTA (sls_uncompressedin));
    mvwprintw (win, y, tx + 56, "%8d", (int) SC_DELTA (sls_compressedin));
    mvwprintw (win, y, tx + 65, "%6d", (int) SC_DELTA (sls_errorin));
    mvwprintw (win, y, tx + 72, "%6d", (int) SC_DELTA (sls_tossed));

    return (1);
}


void DspSlip (WINDOW *win, int ty, int tx)
{
    int     i;
    int     line;

    if (noSyms)
	return;

    DispTimer (win, 0, 24, -1);

    if (slip_num == 0)
	return;

    (void) memcpy (curSlip,
	KTUT (ADDR_slip_data, slip_t *), slip_num * sizeof (slip_t)
    );

    for (line = 0, i = 0; i < slip_num; i++)
	if (ShowSlipEntry (win, ty, tx, line, i))
	    line++;

    if (infoMode == modeDelta)
	(void) memcpy (oldSlip, curSlip, slip_num * sizeof (slip_t));

    wmove (win, getmaxy (win) - 1, getmaxx (win) - 1);
}


void SlipInit ()
{
    noSyms = GetSymbols (kmemSymbols, 0);

    if (noSyms)
	return;

    slip_num = *KTUT (ADDR_slip_num, u_int *);

    if (slip_num > 0)
    {
	curSlip = (slip_t *) malloc (slip_num * sizeof (slip_t));
	if (curSlip == (slip_t *) 0)
	{
	    slip_num = 0;
	    return;
	}

	oldSlip = (slip_t *) malloc (slip_num * sizeof (slip_t));
	if (oldSlip == (slip_t *) 0)
	{
	    slip_num = 0;
	    if (curSlip != (slip_t *) 0)
		free (curSlip);
	    return;
	}

	if (infoMode != modeAbsolute)
	{
	    (void) memcpy (curSlip,
		KTUT (ADDR_slip_data, slip_t *), slip_num * sizeof (slip_t)
	    );
	    (void) memcpy (oldSlip, curSlip, slip_num * sizeof (slip_t));
	}
    }
}

#else

#include <curses.h>
#include <panel.h>

#include "svr4mon.h"
#include "dsputil.h"

void DspSlipLiterals (WINDOW *win, int tly, int tlx)
{
    char    *message = "SLIP statistics not compiled in, sorry!";
    int     x;

    use_cp (win, cpBANNER);
    wmove (win, 0, 0);
    for (x = 0; x < getmaxx (win); x++)
	waddch (win, (chtype) '-');

    use_cp (win, cpBANWARN);
    mvwaddstr (win,
	(getmaxy (win) - 1) / 2,
	(getmaxx (win) - strlen (message)) / 2,
	message
    );

    wclrtoeol (win);
    wclrtobot (win);
}

void DspSlip (WINDOW *win, int tly, int tlx)
{
}

void SlipInit ()
{
}

void SlipModeChange ()
{
}

#endif
