/*
**
**                                 T C P . C
**
**
**      Module:     tcp statistics
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      15-OCT-1992 AV  Version X1.0, first definition
**      16-OCT-1992 AV  display changed values in "bold" mode
**      23-OCT-1992 AV  Version 1.0
**      11-NOV-1993 AV  added MIPS support and LOGGING support
**      14-NOV-1993 AV  split display function in calc in display function
**
*/

#include "config.h"

#if defined(MODULE_TCP)

#include <stdlib.h>
#include <curses.h>
#include <panel.h>

#include <sys/types.h>

#include "tcp.h"

#include "svr4mon.h"
#include "paging.h"
#include "dsputil.h"

/*
**  00000000001111111111222222222233333333334444444444555555555566666666667777777777
**  01234567890123456789012345678901234567890123456789012345678901234567890123456789
**  ---- TCP statistics ------------------------------------------------------------
**  CNattempt ######### keepTMO  ######### rcvtotal  ######### rcvpackAWIN #########
**  accepts   ######### keepPROB ######### rcvpack   ######### rcvbyteAWIN #########
**  connects  ######### keepDROP ######### rcvbyte  ########## rcvafterCLS #########
**  drops     ######### sndtotal ######### rcvbadsum ######### rcvwinprobe #########
**  CNdrops   ######### sndpack  ######### rcvbadoff ######### rcvdupack   #########
**  closed    ######### sndbyte ########## rcvshort  ######### rcvackTMUCH #########
**  segstimed ######### sndRXPAK ######### rcvDPACK  ######### rcvackpack  #########
**  rttUPD    ######### sndRXBYT ######### rcvDBYTE  ######### rcvackbyte  #########
**  delack    ######### sndacks  ######### rcvPDPACK ######### rcvwinupd   #########
**  TMOdrop   ######### sndprobe ######### rcvPDBYTE ######### linger      #########
**  rexmtTMO  ######### sndurg   ######### rcvoopack ######### lingerabort #########
**  prsstTMO  ######### sndwinup ######### rcvoobyte ######### lingerexp   #########
**                      sndctrl  #########                     lingercan   #########
*/

#define ADDR_tcpstat            kmemSymbols[0].n_value

static struct nlist kmemSymbols[] =
{
#if defined(S_SINIX_MIPS)
    { "tcpstat_s"           },
#else
    { "tcpstat"             },
#endif
    { 0                     }
};


#define OLDVAL(ptr)     (*((u_long *) ptr->op))
#define CURVAL(ptr)     (*((u_long *) ptr->cp))
#define OUTVAL(ptr)     (*((u_long *) ptr->outp))

static struct tcpstat   curTcp;
static struct tcpstat   oldTcp;
struct tcpstat          outTcp;

static int              noSyms;

#define DefTcpTag(name)     &curTcp.name,   &oldTcp.name,   &outTcp.name

static struct descrip {
    int         y, tx, vx;
    char        *label;
    char        *fmt;
    u_long      *cp;
    u_long      *op;
    u_long      *outp;
} desc[] = {
    {  1,  0, 10, "CNattempt",   "%9lu",    DefTcpTag (tcps_connattempt) },
    {  2,  0, 10, "accepts",     "%9lu",    DefTcpTag (tcps_accepts) },
    {  3,  0, 10, "connects",    "%9lu",    DefTcpTag (tcps_connects) },
    {  4,  0, 10, "drops",       "%9lu",    DefTcpTag (tcps_drops) },
    {  5,  0, 10, "CNdrops",     "%9lu",    DefTcpTag (tcps_conndrops) },
    {  6,  0, 10, "closed",      "%9lu",    DefTcpTag (tcps_closed) },
    {  7,  0, 10, "segstimed",   "%9lu",    DefTcpTag (tcps_segstimed) },
    {  8,  0, 10, "rttUPD",      "%9lu",    DefTcpTag (tcps_rttupdated) },
    {  9,  0, 10, "delack",      "%9lu",    DefTcpTag (tcps_delack) },
    { 10,  0, 10, "TMOdrop",     "%9lu",    DefTcpTag (tcps_timeoutdrop) },
    { 11,  0, 10, "rexmtTMO",    "%9lu",    DefTcpTag (tcps_rexmttimeo) },
    { 12,  0, 10, "prsstTMO",    "%9lu",    DefTcpTag (tcps_persisttimeo) },

    {  1, 20, 29, "keepTMO",     "%9lu",    DefTcpTag (tcps_keeptimeo) },
    {  2, 20, 29, "keepPROB",    "%9lu",    DefTcpTag (tcps_keepprobe) },
    {  3, 20, 29, "keepDROP",    "%9lu",    DefTcpTag (tcps_keepdrops) },
    {  4, 20, 29, "sndtotal",    "%9lu",    DefTcpTag (tcps_sndtotal) },
    {  5, 20, 29, "sndpack",     "%9lu",    DefTcpTag (tcps_sndpack) },
    {  6, 20, 28, "sndbyte",     "%10lu",   DefTcpTag (tcps_sndbyte) },
    {  7, 20, 29, "sndRXPAK",    "%9lu",    DefTcpTag (tcps_sndrexmitpack) },
    {  8, 20, 29, "sndRXBYT",    "%9lu",    DefTcpTag (tcps_sndrexmitbyte) },
    {  9, 20, 29, "sndacks",     "%9lu",    DefTcpTag (tcps_sndacks) },
    { 10, 20, 29, "sndprobe",    "%9lu",    DefTcpTag (tcps_sndprobe) },
    { 11, 20, 29, "sndurg",      "%9lu",    DefTcpTag (tcps_sndurg) },
    { 12, 20, 29, "sndwinup",    "%9lu",    DefTcpTag (tcps_sndwinup) },
    { 13, 20, 29, "sndctrl",     "%9lu",    DefTcpTag (tcps_sndctrl) },

    {  1, 39, 49, "rcvtotal",    "%9lu",    DefTcpTag (tcps_rcvtotal) },
    {  2, 39, 49, "rcvpack",     "%9lu",    DefTcpTag (tcps_rcvpack) },
    {  3, 39, 48, "rcvbyte",     "%10lu",   DefTcpTag (tcps_rcvbyte) },
    {  4, 39, 49, "rcvbadsum",   "%9lu",    DefTcpTag (tcps_rcvbadsum) },
    {  5, 39, 49, "rcvbadoff",   "%9lu",    DefTcpTag (tcps_rcvbadoff) },
    {  6, 39, 49, "rcvshort",    "%9lu",    DefTcpTag (tcps_rcvshort) },
    {  7, 39, 49, "rcvDPACK",    "%9lu",    DefTcpTag (tcps_rcvduppack) },
    {  8, 39, 49, "rcvDBYTE",    "%9lu",    DefTcpTag (tcps_rcvdupbyte) },
    {  9, 39, 49, "rcvPDPACK",   "%9lu",    DefTcpTag (tcps_rcvpartduppack) },
    { 10, 39, 49, "rcvPDBYTE",   "%9lu",    DefTcpTag (tcps_rcvpartdupbyte) },
    { 11, 39, 49, "rcvoopack",   "%9lu",    DefTcpTag (tcps_rcvoopack) },
    { 12, 39, 49, "rcvoobyte",   "%9lu",    DefTcpTag (tcps_rcvoobyte) },

    {  1, 59, 71, "rcvpackAWIN", "%9lu",    DefTcpTag (tcps_rcvpackafterwin) },
    {  2, 59, 71, "rcvbyteAWIN", "%9lu",    DefTcpTag (tcps_rcvbyteafterwin) },
    {  3, 59, 71, "rcvafterCLS", "%9lu",    DefTcpTag (tcps_rcvafterclose) },
    {  4, 59, 71, "rcvwinprobe", "%9lu",    DefTcpTag (tcps_rcvwinprobe) },
    {  5, 59, 71, "rcvdupack",   "%9lu",    DefTcpTag (tcps_rcvdupack) },
    {  6, 59, 71, "rcvackTMUCH", "%9lu",    DefTcpTag (tcps_rcvacktoomuch) },
    {  7, 59, 71, "rcvackpack",  "%9lu",    DefTcpTag (tcps_rcvackpack) },
    {  8, 59, 71, "rcvackbyte",  "%9lu",    DefTcpTag (tcps_rcvackbyte) },
    {  9, 59, 71, "rcvwinupd",   "%9lu",    DefTcpTag (tcps_rcvwinupd) },

#if defined(NETWORK_WOLLONGONG) && defined(SNMP)
    { 10, 59, 71, "retransseg",  "%9lu",    DefTcpTag (tcpRetransSegs) },
#elif defined(NETWORK_LACHMAN)
    { 10, 59, 71, "linger",      "%9lu",    DefTcpTag (tcps_linger) },
    { 11, 59, 71, "lingerabort", "%9lu",    DefTcpTag (tcps_lingerabort) },
    { 12, 59, 71, "lingerexp",   "%9lu",    DefTcpTag (tcps_lingerexp) },
    { 13, 59, 71, "lingercan",   "%9lu",    DefTcpTag (tcps_lingercan) },
#endif

};


void TcpVarInit (void)
{
    if ((noSyms = GetSymbols (kmemSymbols, 0)))
	return;

    if (infoMode != modeAbsolute)
    {
	(void) memcpy (&curTcp,
	    KTUT (ADDR_tcpstat, struct tcpstat *), sizeof (struct tcpstat)
	);
	(void) memcpy (&oldTcp, &curTcp, sizeof (struct tcpstat));
    }
}


void TcpDspInit (WINDOW *win, int tly, int tlx)
{
    static char     *header = "---- TCP statistics --- ";
    int             x;
    int             y;
    struct descrip  *dptr;

    wmove (win, tly, tlx);

    use_cp (win, cpBANNER);
    waddstr (win, header);
    getyx (win, y, x);
    while (x++ < getmaxx (win))
	waddch (win, (chtype) '-');

    if (noSyms)
    {
	char    *message = "TCP related kernel variables not found!";
	int     x;

	use_cp (win, cpBANWARN);
	mvwaddstr (win,
	    (getmaxy (win) - 1) / 2,
	    (getmaxx (win) - strlen (message)) / 2,
	    message
	);

	wclrtoeol (win);
	wclrtobot (win);
	return;
    }

    use_cp (win, cpLIT);

    for (x = 0, dptr = desc; x < sizeof (desc) / sizeof (desc[0]); dptr++, x++)
	mvwaddstr (win, tly + dptr->y, tlx + dptr->tx, dptr->label);
}


void TcpPreCalc (void)
{
    struct descrip  *dptr;
    int             i;

    if (noSyms)
	return;

    (void) memcpy (&curTcp,
	KTUT (ADDR_tcpstat, struct tcpstat *), sizeof (struct tcpstat)
    );

    for (i = 0, dptr = desc; i < sizeof (desc) / sizeof (desc[0]); dptr++, i++)
    {
	if (dptr->fmt == 0 || dptr->fmt[0] == 0)
	    continue;

	if (infoMode == modeAbsolute)
	    OUTVAL (dptr) = CURVAL (dptr);
	else
	    OUTVAL (dptr) = CURVAL (dptr) - OLDVAL (dptr);
    }
}


void TcpDisplay (WINDOW *win, int tly, int tlx)
{
    struct descrip  *dptr;
    int             i;
    int             flag;

    if (noSyms)
	return;

    DispTimer (win, 0, 23, -1);

    use_cp (win, cpINFO);

    for (i = 0, dptr = desc; i < sizeof (desc) / sizeof (desc[0]); dptr++, i++)
    {
	if (dptr->fmt == 0 || dptr->fmt[0] == 0)
	    continue;

	if (infoMode == modeAbsolute)
	    mvwprintw (win, tly + dptr->y, tlx + dptr->vx, dptr->fmt, OUTVAL (dptr));
	else
	{
	    if ((flag = (CURVAL (dptr) != OLDVAL (dptr))))
		use_cp (win, cpHIGH);
	    mvwprintw (win, tly + dptr->y, tlx + dptr->vx, dptr->fmt, OUTVAL (dptr));
	    if (flag)
		use_cp (win, cpINFO);
	}
    }

    wmove (win, getmaxy (win) - 1, getmaxx (win) - 1);
}


void TcpPostCalc (void)
{
    if (noSyms)
	return;

    if (infoMode == modeDelta)
	(void) memcpy (&oldTcp, &curTcp, sizeof (struct tcpstat));
}


void TcpModeChange (void)
{
    if (noSyms)
    {
	beep ();
	return;
    }

    switch (infoMode)
    {
	case modeDelta:
	case modeRelative:
	    (void) memcpy (&oldTcp, &curTcp, sizeof (struct tcpstat));
	    break;
	case modeAbsolute:
	    break;
    }
}


void TcpExit (void)
{
}


#else

#include <curses.h>
#include <panel.h>

#include "svr4mon.h"
#include "dsputil.h"

void TcpVarInit (void) {}

void TcpDspInit (WINDOW *win, int tly, int tlx)
{
    char    *message = "TCP statistics not compiled in, sorry!";
    int     x;

    use_cp (win, cpBANNER);
    wmove (win, 0, 0);
    for (x = 0; x < getmaxx (win); x++)
	waddch (win, (chtype) '-');

    use_cp (win, cpBANWARN);
    mvwaddstr (win,
	(getmaxy (win) - 1) / 2,
	(getmaxx (win) - strlen (message)) / 2,
	message
    );

    wclrtoeol (win);
    wclrtobot (win);
}

void TcpPreCalc (void) {}
void TcpDisplay (WINDOW *win, int tly, int tlx) {}
void TcpPostCalc (void) {}
void TcpModeChange (void) {}
void TcpExit (void) {}

#endif
