/*
**
**                             U I D U T I L . C
**
**
**      Module:     uid hashing functions
**      Version:    X1.0
**      Author:     Andreas Vogel (AV)
**      SccsId:     %Z%%M% %I% %G%
**
**  Copyright (C) 1992,1993  Andreas Vogel <av@ssw.de>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  Modification history:
**
**      09-OCT-1992 AV      Version X1.0, first definition
**      17-OCT-1992 AV      dependency on patchlevel.h not needed
**      23-OCT-1992 AV      Version 1.0
**
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <libgen.h>
#include <pwd.h>

#include <sys/types.h>
#include <sys/utsname.h>

#define UID_HASH_SIZE   127
#define HASH_EMPTY      ((uid_t) -1)
#define HASHIT(i)       ((i) % UID_HASH_SIZE)

struct UidHashEntry
{
    uid_t   uid;
    char    name[64];
};

static struct UidHashEntry  uidHashTable[UID_HASH_SIZE];
static int                  uidHashCount = 0;

void InitUidToNameHash (void)
{
    int                 ihash = 0;
    struct UidHashEntry *hashent = uidHashTable;

    while (ihash++ < UID_HASH_SIZE)
    {
	hashent->uid = HASH_EMPTY;
	hashent++;
    }
}

static int UidNameEnter (uid_t uid, char *name)
{
    uid_t   table_uid;
    int     hashval;

    if (++uidHashCount >= UID_HASH_SIZE - 1)
	LeaveText ("too many user names for me to handle", 1);

    hashval = HASHIT (uid);

    while ((table_uid = uidHashTable[hashval].uid) != HASH_EMPTY)
    {
	if (table_uid == uid)
	    return (hashval);
	hashval = (hashval + 1) % UID_HASH_SIZE;
    }

    uidHashTable[hashval].uid = uid;

    (void) strncpy (
	uidHashTable[hashval].name, name, sizeof (uidHashTable[0].name)
    );

    return (hashval);
}

static int GetPwentAndEnter (uid_t uid)
{
    int             hashval;
    struct passwd   *pwd;
    char                    errant[64];

    pwd = getpwuid (uid);
    endpwent ();

    if (pwd)
    {
	hashval = UidNameEnter (pwd->pw_uid, pwd->pw_name);
	return (hashval);
    }

    (void) sprintf (errant, "%u", uid);

    return (UidNameEnter (uid, errant));
}

char *UidToName (uid_t uid)
{
    int     uid_hash;
    uid_t   table_uid;

    uid_hash = HASHIT (uid);

    while ((table_uid = uidHashTable[uid_hash].uid) != uid)
    {
	if (table_uid == HASH_EMPTY)
	{
	    /* not in hash table */
	    uid_hash = GetPwentAndEnter (uid);
	    break;
	}
	uid_hash = (uid_hash + 1) % UID_HASH_SIZE;
    }

    return (uidHashTable[uid_hash].name);
}
